package applications;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

// oeffnungsort wird nicht uebernommen


//gui imports
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;

import java.util.Vector;

//event imports
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.imageio.ImageIO;
import java.io.File;

//library imports
import standard.ImageSequence;
import gui.AbstractImageEffectPanel;
import gui.ImageEffectSaveOption;
import gui.AbstractPropertiesPanel;
import gui.SimpleImageEffectPanel;
import gui.fractals.MandelbrotSetPanel;
import gui.drawing.StarsPanel;
import gui.ImageEffectInternalFrame;
import gui.PropertiesInternalFrame;
import color.GrayscaleReduction;
import color.NoiseGenerator;
import color.ColorTransition;
import color.ColorTransitionParameters;
import transformation.ImageResize;
import transformation.ImageResizeParameters;
import drawing.Network;
import drawing.BrickTexture;

import static applications.LanguageProperties.*;
import static applications.Settings.*;

/**
 *  The starting point and main frame of the gui application of the JFXLibrary.
 **/

public class GuiApplication extends JFrame implements ISettingsListener {

  /*** member variables ***/
  private JDesktopPane m_pDesktop;

  private LanguageProperties m_pLanguageProperties = null;
  private GuiApplicationMenu m_pMenu;


  Vector<ImageEffectInternalFrame> m_pInternalFrames = null;
  ImageEffectInternalFrame m_pCurrentInternalFrame = null;
  PropertiesInternalFrame m_pPropertiesFrame = null;

  SettingsDialog m_pSettingsDialog = null;


  /*** new gui members ***/
  AbstractImageEffectPanel m_pCurrentImageEffectPanel = null;
  AbstractPropertiesPanel m_pCurrentPropertiesPanel = null;




  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates the main frame of the GUI application.
   **/
  public GuiApplication() {

    //1. init the frame
    super();

    //2. Set up the GUI.
    //m_pDesktop = new JDesktopPane(); //a specialized layered pane
    //setContentPane(m_pDesktop);
    m_pMenu = new GuiApplicationMenu(this);
    setJMenuBar(m_pMenu.getMenuBar());
    setLayout(new BorderLayout());



    //3. settings
    try {
      m_pSettingsDialog = new SettingsDialog(this);
      m_pSettingsDialog.addSettingsListener(this);
      m_pSettingsDialog.addSettingsListener(m_pMenu);
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }



    //4. set the size and position of the frame
    //   Make the big window be indented 50 pixels from each edge
    //   of the screen.
    int iInset = 50;
    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iWindowWidth = pScreenSize.width  - iInset*2;
    int iWindowHeight = pScreenSize.height - iInset*2;
    setBounds(iInset, iInset, iWindowWidth, iWindowHeight);
    setExtendedState(MAXIMIZED_BOTH); //set application maximized





    /*
    //5. create the properties frame
    int iPropFrameWidth = 300;
    int iPropFrameHeight = iWindowHeight - 100;
    m_pPropertiesFrame = new PropertiesInternalFrame(iPropFrameWidth, iPropFrameHeight);
    m_pPropertiesFrame.setLocation(iWindowWidth - iPropFrameWidth, 0);
    try {
      m_pDesktop.add(m_pPropertiesFrame);
      m_pPropertiesFrame.setSelected(true);
      m_pPropertiesFrame.setVisible(true);
    }
    catch (Exception pException) {}
    */


    //finally apply the settings
    applyNewSettings(SettingsDialog.getSettings());
    m_pMenu.applyNewSettings(SettingsDialog.getSettings());

    if (m_pSettingsDialog.needsInitialization()) {
      m_pSettingsDialog.setVisible(true);
    }




  }




  //////////////////////////
  // SET METHODS          //
  //////////////////////////
  /**
   *  Sets the panel of the effect that should be shown now.
   *
   *  @param pImageEffectInternalFrame The effect panel that should be shown now.
   **/
  public void setCurrentEffectPanel(ImageEffectInternalFrame pImageEffectInternalFrame) {
    if (pImageEffectInternalFrame == null) return;
    m_pCurrentInternalFrame = pImageEffectInternalFrame;
  }




  ////////////////////////////////////
  // ISettingsListener METHODS      //
  ////////////////////////////////////

  /**
   *  Apply the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettings(Settings pSettings) {
    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    m_pLanguageProperties = pSettings.getLanguageProperties();

    //3. set the labels for the application
    setTitle(m_pLanguageProperties.getProperty(KEY_APP_NAME));



  }

  /**
   *  Sets the image effect panel that should be shown now.
   *
   *  @param pTitle The title of the image effect as string.
   *  @param pPanel The image effect panel object.
   **/
  public void setImageEffectPanel(String pTitle, AbstractImageEffectPanel pPanel) {
    //1. make some checks
    if (pPanel == null) return;
    if (pTitle == null) pTitle = "";
    pTitle = pTitle.trim();

    //2. remove the old image effect panel
    if (m_pCurrentImageEffectPanel != null) {
      remove(m_pCurrentImageEffectPanel);
    }

    //3. set the new image effect panel
    add(pPanel, BorderLayout.CENTER);
    m_pCurrentImageEffectPanel = pPanel;
    pPanel.revalidate();

    //4. set the title
    setTitle(m_pLanguageProperties.getProperty(LanguageProperties.KEY_APP_NAME) + " - " + pTitle);

    //5. set the properties panel
    setPropertiesPanel(m_pCurrentImageEffectPanel.getPropertiesPanel());

    m_pSettingsDialog.addSettingsListener(pPanel);

  }


  /**
   *  Sets the properties panel that should be shown now to steer the parameters of the current effect.
   *
   *  @param pPropertiesPanel The effect properties panel that should be shown to steer the effect parameters.
   **/
  public void setPropertiesPanel(AbstractPropertiesPanel pPropertiesPanel) {
    //1. make a simple check
    if (pPropertiesPanel == null) return;

    //2. remove the old panel
    if (m_pCurrentPropertiesPanel != null) {
      remove(m_pCurrentPropertiesPanel);
    }

    //3. set the new panel
    m_pCurrentPropertiesPanel = pPropertiesPanel;
    add(m_pCurrentPropertiesPanel, BorderLayout.SOUTH);
    m_pCurrentPropertiesPanel.setVisible(m_pMenu.showProperties());
    m_pCurrentPropertiesPanel.revalidate(); //is needed to show the new panel immediately.

    //4. add the panel to the setting listener
    m_pSettingsDialog.addSettingsListener(m_pCurrentPropertiesPanel);



  }

  public void saveCurrentEffectImage() {
    //I. fetch the frame that has the focus
    boolean bNoImageAvailable = (m_pCurrentImageEffectPanel == null);

    if (bNoImageAvailable) {
      //to do: put those messages into the config file
      String pError = m_pLanguageProperties.getProperty(KEY_ERROR_NO_IMAGE_TO_SAVE);
      String pTitle = m_pLanguageProperties.getProperty(KEY_ERROR);
      JOptionPane.showMessageDialog(this, pError, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }

    SaveOptionsDialog saveOptionsDialog = new SaveOptionsDialog(this, m_pCurrentImageEffectPanel.getSaveOptions());
    if (saveOptionsDialog.shouldBeShown()) {
      saveOptionsDialog.setVisible(true);
    }

    if (saveOptionsDialog.aborted()) return;

    //II. get the image of this frame
    BufferedImage pImage = null;
    try {
      pImage = m_pCurrentImageEffectPanel.getEffectImageForSaving(saveOptionsDialog.getSaveOptions());
    }
    catch (Exception pException) {
      String error = pException.getMessage();
      String pTitle = m_pLanguageProperties.getProperty(KEY_ERROR);
      String translatedError = m_pLanguageProperties.getProperty(error);
      JOptionPane.showMessageDialog(this, translatedError, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }

    //III. save the image
    JFileChooser pChooser = new JFileChooser();
    FileNameExtensionFilter pFilter = new FileNameExtensionFilter("Image files", "png", "bmp", "gif", "jpg", "jpeg");
    pChooser.setFileFilter(pFilter);
    File pCurrentDirectory = null;
    Settings pSettings = SettingsDialog.getSettings();
    try {
      pCurrentDirectory = new File(pSettings.getProperty(PROP_KEY_STORE_IMAGE_DIR));
    }
    catch (Exception pException) {}
    pChooser.setCurrentDirectory(pCurrentDirectory);

    String pCurrentFileName = null;
    if (m_pCurrentPropertiesPanel != null) pCurrentFileName = m_pCurrentPropertiesPanel.getFileName();
    if (pCurrentFileName != null) pChooser.setSelectedFile(new File(pCurrentFileName));

    try {
      int iResult = pChooser.showSaveDialog(this);
      if (iResult == JFileChooser.APPROVE_OPTION) {
        File pFile = pChooser.getSelectedFile();
        ImageSequence.saveImage(pImage, pFile);

        pSettings.setProperty("StoreImage.directory", pChooser.getCurrentDirectory().getAbsolutePath());
        pSettings.store(SettingsDialog.SETTINGS_PROPERTY_FILE);
      }
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }
  }

  public void showSettingsDialog() {
   try {
      m_pSettingsDialog.setVisible(true);
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }
  }

  public void showPropertiesPanel(boolean bShow) {
    //m_pPropertiesFrame.setVisible(m_pViewPropertiesFrame.isSelected());
    if (m_pCurrentPropertiesPanel != null) m_pCurrentPropertiesPanel.setVisible(bShow);
  }



  private void createFrame(String pTitle, AbstractImageEffectPanel pPanel) {

    //0. check if there is a panel
    if (pPanel == null) return;

    //1. check if the data structure has to be initiated
    if (m_pInternalFrames == null) m_pInternalFrames = new Vector<ImageEffectInternalFrame>();

    //2. create the internal frame
    ImageEffectInternalFrame pFrame = new ImageEffectInternalFrame(pTitle, pPanel, m_pPropertiesFrame, this);
    pFrame.setSize(new Dimension(getWidth() - m_pPropertiesFrame.getSize().width, getHeight() - 100));
    pFrame.setLocation(0, 0);
    pFrame.setVisible(true);


    //3. show the internal frame and set the image effect picture
    try {
      m_pDesktop.add(pFrame);
      pFrame.setSelected(true);
    }
    catch (Exception pException) {}



    //4. add the frame to the data structure
    m_pInternalFrames.add(pFrame);
  }


  ////////////////////////////////
  // START METHOD               //
  ////////////////////////////////

  /**
   *  Starts the GUI application.
   *
   *  @param args A string array to pass on parameters. But it is not used at the moment.
  **/
  public static void main(String[] args) {

    //Make sure we have nice window decorations.
    try {
	    // Set System L&F
        UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception e) {
       // handle exception
    }

    //JFrame.setDefaultLookAndFeelDecorated(true);


    //Create and set up the window.
    GuiApplication pFrame = new GuiApplication();
    pFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    //Display the window.
    pFrame.setVisible(true);

  }







}
