package applications;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileInputStream;

import java.util.Properties;
import java.util.Vector;
import java.util.Iterator;

/**
 * An entity that holds the language properties, for example
 * all label names for the specified language.
 **/

public class LanguageProperties {


  /*** CONSTANTS ***/
  /** The string constant for the language English. **/
  public static final String ENGLISH = "ENGLISH";
  /** The string constant for the language German. **/
  public static final String GERMAN = "GERMAN";
  /** The string constant for the language Austrian. **/
  public static final String AUSTRIAN = "AUSTRIAN";

  //GENERAL TRANSLATIONS
  public static final String KEY_APP_NAME = "AppName";
  public static final String KEY_OK = "OK";
  public static final String KEY_ABORT = "Abort";
  public static final String KEY_FILE_MENU = "FileMenu";
  public static final String KEY_SAVE_IMAGE = "SaveImage";
  public static final String KEY_EXIT = "Exit";

  //VIEW MENU
  public static final String KEY_VIEW_MENU = "ViewMenu";
  public static final String KEY_VIEW_PROPERTIES_FRAME = "ViewPropertiesFrame";

  //IMAGE EFFECT MENU
  public static final String KEY_IMAGE_EFFECT_MENU = "ImageEffectMenu";
  public static final String KEY_TRANSFORMATION_MENU = "TransformationMenu";
  public static final String KEY_FRACTALS_MENU = "FractalsMenu";
  public static final String KEY_MANDELBROT_SET_MENU = "MandelbrotSetMenu";
  public static final String KEY_JULIA_SET_MENU = "JuliaSetMenu";
  public static final String KEY_JULIA_SET2_MENU = "JuliaSet2Menu";
  public static final String KEY_RESIZE_MENU = "ResizeMenu";
  public static final String KEY_DRAWING_MENU = "DrawingMenu";
  public static final String KEY_STARS_MENU = "StarsMenu";
  public static final String KEY_NETWORK_MENU = "NetworkMenu";
  public static final String KEY_BRICK_TEXTURE_MENU = "BrickTextureMenu";
  public static final String KEY_COLOR_MENU = "ColorMenu";
  public static final String KEY_GRAYSCALE_REDUCTION_MENU = "GrayscaleReductionMenu";
  public static final String KEY_COLOR_TRANSITION_MENU = "ColorTransitionMenu";
  public static final String KEY_NOISE_GENERATOR_MENU = "NoiseGeneratorMenu";
  public static final String KEY_TRANSPARENCY_MENU = "TransparencyMenu";
  public static final String KEY_IMAGE_TRANSITION_MENU = "ImageTransitionMenu";
  public static final String KEY_CONFIGURATION_MENU = "ConfigurationMenu";
  public static final String KEY_SETTINGS_MENU = "SettingsMenu";


  public static final String KEY_SAVE_OPTIONS_DIALOG_TITLE = "SaveOptionsDialogTitle";

  //ERRORS
  public static final String KEY_ERROR = "Error";
  private static final String ERROR_KEY_PREFIX = "Error.";
  public static final String KEY_ERROR_NO_IMAGE_TO_SAVE = ERROR_KEY_PREFIX + "NoImageToSave";





  /*** member variables ***/
  private Properties m_pProperties;
  private DefaultLanguageProperties m_pFallbackProperties;

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates an object with the labels of the specified language. If the specified language configuration
   *  cannot be loaded, the {@code DefaultLanguageProperties} are used to return the English default labels.
   *
   *  @param pLanguage The language of the properties as string.
   *
   **/
  public LanguageProperties(String pLanguage) {


    //1. fetch the file path
    String pFileName = getFileName(pLanguage);

    //2. load the properties
    try {
      m_pProperties = new Properties();
      BufferedInputStream pStream = new BufferedInputStream(new FileInputStream(pFileName));
      m_pProperties.load(pStream);
      pStream.close();
    }
    catch (Exception pException) {
      System.out.println("The property file " + pFileName +
      " could not be loaded! " + pException.getMessage());
    }

    //3. fallback properties
    m_pFallbackProperties = new DefaultLanguageProperties();
    checkMissingProperties(pLanguage);

  }


  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the value of the specified property.
   *
   *  @param pProperty A string indicating the name of the property.
   *
   *  @return The value of the specified property as string.
   **/
  public String getProperty(String pProperty) {
    //1. make a simple check
    if (pProperty == null) return null;
    if (pProperty.trim().equals("")) return null;

    //2. return the language property
    if (m_pProperties != null) {
      String pValue = m_pProperties.getProperty(pProperty);
      if (pValue != null) return pValue;
    }

    //3. if not available return the value from the fallback properties
    return m_pFallbackProperties.getProperty(pProperty);
  }


  public Vector<String> getPropertyNames() {
    if (m_pProperties == null) return new Vector<String>();

    Iterator<String> pPropertyNames = m_pProperties.stringPropertyNames().iterator();
    Vector<String> pVector = new Vector<String>();
    while (pPropertyNames.hasNext()) {
      pVector.add(pPropertyNames.next());
    }
    return pVector;
  }



  //////////////////////////////
  // STATIC HELPER METHODS    //
  //////////////////////////////
  private static String getFileName(String pLanguage) {
    //1. make an initial check
    if ((pLanguage == null) || (pLanguage.trim().equals(""))) pLanguage = ENGLISH;

    //2. return the path for the given language
    String pPrefix = "props" + File.separator + "languages" + File.separator;
    if (pLanguage.equalsIgnoreCase(GERMAN)) {
      return pPrefix + "german.properties";
    }
    else if (pLanguage.equalsIgnoreCase(AUSTRIAN)) {
      return pPrefix + "austrian.properties";
    }

    return pPrefix + "english.properties";
  }


  //////////////////////////////
  // PRIVATE HELPER METHODS   //
  //////////////////////////////
  private void checkMissingProperties(String pLanguage) {
    Vector<String> pAllNames = m_pFallbackProperties.getPropertyNames();
    Vector<String> pMyNames = getPropertyNames();

    System.out.println("MISSING PROPERTIES CHECK FOR LANGUAGE " + pLanguage);

    int iSize = pAllNames.size();
    int iMissingCount = 0;
    for (int i = 0; i < iSize; i++) {
      String pKey = pAllNames.get(i);
      if (pMyNames.contains(pKey)) {
        pMyNames.remove(pKey);
      }
      else {
        iMissingCount++;
        System.out.println("KEY: " + pKey + " is missing!");
      }
    }

    if (iMissingCount == 0) {
      System.out.println("ALL PROPERTIES CONTAINED - EVERYTHING OK");
    }
    else {
      System.out.println("[ERROR]: " + iMissingCount + " MISSING PROPERTIES DETECTED!!!");
    }

    int iWrongNamesCount = pMyNames.size();
    if (iWrongNamesCount > 0) {
      System.out.println("THE FOLLOWING " + iWrongNamesCount + " PROPERTIES ARE INVALID!");
      for (int i = 0; i < iWrongNamesCount; i++) {
        System.out.println(pMyNames.get(i));
      }
    }
  }

}
