package applications;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Properties;
import java.util.Vector;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;

import javax.swing.*;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


import static applications.LanguageProperties.*;
import static applications.Settings.*;

/***
 *   Dialog that represents the options of the gui application.
 **/

public class SettingsDialog extends JDialog implements ActionListener {

  /*** translation constants ***/
  private static final String SETTINGS_KEY_PREFIX = "Settings.";
  public static final String LANG_KEY_LANGUAGE = SETTINGS_KEY_PREFIX + "Language";

  /** private member variables **/
  private static Settings m_pSettings;
  private Vector<ISettingsListener> m_pSettingsListener = null;

  /*** private gui member variables **/
  private JComboBox<String> m_pLanguageCombo;
  private JButton m_pOk, m_pAbort;
  private JLabel m_pLanguageLabel;


  /*** constants ***/
  /*** The file name of the java properties file where the options of the gui application are stored. ***/
  public static final String SETTINGS_PROPERTY_FILE = "props" + File.separator + "settings.properties";

  /////////////////////////////////
  // Constructor                 //
  /////////////////////////////////
  /**
   *  Creates the settings dialog.
   *
   *  @param pOwner The {@code JFrame} object that is blocked while the settings dialog is shown.
   *
   *  @throws Exception If the application settings could not be loaded.
   **/
  public SettingsDialog(JFrame pOwner) throws Exception {
    //1. init the dialog and make a check
    super(pOwner);

    //TODO: Check why the Austrian abort button is not shown!!!

    //2. get the settings
    m_pSettings = getSettings();
    LanguageProperties pLanguageProperties = m_pSettings.getLanguageProperties();

    //3. set the layout
    setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

    JPanel pLanguagePanel = new JPanel();
    pLanguagePanel.setLayout(new FlowLayout());
    m_pLanguageLabel = new JLabel(pLanguageProperties.getProperty(LANG_KEY_LANGUAGE));
    pLanguagePanel.add(m_pLanguageLabel);

    Vector<String> pLanguages = new Vector<String>();
    pLanguages.add("English");
    pLanguages.add("German");
    pLanguages.add("Austrian");
    m_pLanguageCombo = new JComboBox<String>(pLanguages);
    m_pLanguageCombo.setSelectedItem(m_pSettings.getProperty(PROP_KEY_LANGUAGE));
    pLanguagePanel.add(m_pLanguageCombo);




    //4. ok and abort button
    JPanel pOkPanel = new JPanel();
    pOkPanel.setLayout(new FlowLayout());

    m_pOk = new JButton(pLanguageProperties.getProperty(KEY_OK));
    m_pOk.addActionListener(this);
    pOkPanel.add(m_pOk);

    m_pAbort = new JButton(pLanguageProperties.getProperty(KEY_ABORT));
    m_pAbort.addActionListener(this);
    pOkPanel.add(m_pAbort);

    //5. add the panels
    add(pLanguagePanel);
    add(pOkPanel);


    //6. set size and the location of the panel (= center of the screen)
    /*
    int iWidth = 200;
    int iHeight = 120;
    setSize(new Dimension(iWidth,iHeight));
    */

    pack(); // sets the optimum size to the dialog

    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iXPos = (pScreenSize.width-getWidth())/2;
    int iYPos = (pScreenSize.height-getHeight())/2;
    setLocation(iXPos, iYPos);


  }


  ///////////////////////////////////
  // GET METHODS                   //
  ///////////////////////////////////
  /**
   *  Returns the {@code Settings} object that is used by the application. As there can be only one
   *  Setting for the application, this is a static method.
   *
   *  @return The {@code Settings} object that determines the configuration of the gui application
   **/
  public static Settings getSettings() {
    if (m_pSettings != null) return m_pSettings;

    //1. load the properties
    try {
      Properties pProperties = new Properties();
      BufferedInputStream pStream = new BufferedInputStream(new FileInputStream(SETTINGS_PROPERTY_FILE));
      pProperties.load(pStream);
      pStream.close();
      return new Settings(pProperties);
    }
    catch (Exception pException) {
      System.out.println("The property file " + SETTINGS_PROPERTY_FILE +
      " could not be loaded! " + pException.getMessage());
    }

    //2. fallback properties
    return new Settings();
  }

  /**
   *  Returns true, if the user has to make some first initializations of the Gui application.
   *
   *
   *  @return true, if the user has to make some first initializations of the Gui application.
   **/
  public boolean needsInitialization() {
    return m_pSettings.needsInitialization();
  }




  ////////////////////////////////////
  // SET METHODS                    //
  ////////////////////////////////////
  /**
   *  Adds a {@code ISettingsListener} object to a list of listeners.
   *  Every time when a change in the Settings dialog is made, all listeners get notified and can react accordingly.
   *
   *  @param pSettingsListener The {@code ISettingsListener} object that should be notified, when the dialog changes settings.
   **/
  public void addSettingsListener(ISettingsListener pSettingsListener) {
    //1. make an initial check
    if (pSettingsListener == null) return;

    //2. init the data structure if necessary
    if (m_pSettingsListener == null) {
      m_pSettingsListener = new Vector<ISettingsListener>();
    }

    //3. nothing to do, if the current listener is already part of the list
    if (m_pSettingsListener.contains(pSettingsListener)) return;

    //3. add
    m_pSettingsListener.add(pSettingsListener);


  }





  ////////////////////////////////////////
  // ACTIONLISTENER METHODS             //
  ////////////////////////////////////////

  /**
   *  Implementation of the {@code ActionListener} interface to handle button events.
   *
   *  @param e The {@code ActionEvent} object that is used to get the source of the action.
   **/
  public void actionPerformed(ActionEvent e) {
    Object pSource = e.getSource();

    //just close the dialog -> ABORT
    if (pSource.equals(m_pAbort)) {
      setVisible(false);
    }


    //save the settings, apply all settings to the listeners, close the dialog
    if (pSource.equals(m_pOk)) {
      //I. save the settings
      storeSettings();

      //II. apply the settings to the listeners
      applyNewSettings();


      //III. close the funny dialog
      setVisible(false);
    }


  }



  //////////////////////////////////////
  // PRIVATE HELPER METHODS           //
  //////////////////////////////////////
  private void storeSettings() {
    //1. set the settings
    try {
      m_pSettings.setProperty("Language", (String)m_pLanguageCombo.getSelectedItem());
    }
    catch (Exception pException) {
      System.out.println("ERROR: Set the settings! (funny log - ugly error)");
      pException.printStackTrace();
    }

    //2. store it
    try {
      m_pSettings.store(SETTINGS_PROPERTY_FILE);
    }
    catch (Exception pException) {
      System.out.println("ERROR: Storing the settings!");
      pException.printStackTrace();
    }
  }

  private void applyNewSettings() {
    //1. make some very simple checks
    if (m_pSettingsListener == null) return;
    if (m_pSettingsListener.size() == 0) return;

    //2. run through all listeners
    int iListenerNum = m_pSettingsListener.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pSettingsListener.get(i).applyNewSettings(m_pSettings);
    }

    //3. set the language settings for this dialog
    LanguageProperties pLanguageProperties = m_pSettings.getLanguageProperties();
    m_pOk.setText(pLanguageProperties.getProperty(KEY_OK));
    m_pAbort.setText(pLanguageProperties.getProperty(KEY_ABORT));
    m_pLanguageLabel.setText(pLanguageProperties.getProperty(LANG_KEY_LANGUAGE));

  }

}
