package color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;
import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.IntegerParameter;
import standard.ColorParameter;
import standard.ObjectParameter;
import standard.BooleanParameter;
import util.image.ExampleImageGenerator;

/**
 *   Encapsulates the parameters that can be specified for the {@code ColorTransition} effect object. 
 **/
public class ColorTransitionParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: transition length in number of pixels  ***/
  public static final String PARAM_INT_TRANSITION_LENGTH = "TRANSITION_LENGTH";
  
  /*** KEY for the color parameter: transition color  ***/
  public static final String PARAM_COLOR_TRANSITION_COLOR = "TRANSITION_COLOR";
  
  /*** KEY for the object parameter: image ***/
  public static final String PARAM_OBJECT_IMAGE = "IMAGE";
  
  /*** KEY for the boolean parameter: transition top ***/
  public static final String PARAM_BOOL_TRANSITION_TOP = "TRANSITION_TOP";
  
  /*** KEY for the boolean parameter: transition bottom ***/
  public static final String PARAM_BOOL_TRANSITION_BOTTOM = "TRANSITION_BOTTOM";
    
   /***
   *   Creates a default parameter set for the {@code ColorTransition} effect. The following settings are used:
   *
   *   <ul>
   *     <li>A transition length of 80 pixels.</li>
   *     <li>A transition end color of rgb(50, 50, 255)</li>
   *     <li>An empty image with the dimension 400 x 400 pixels</li>
   *     <li>A transition to the bottom is activated.</li>
   *   </ul>
   *
   **/
  public ColorTransitionParameters() {

    //1. create the parameters and set them
    setIntegerParameters(createIntegerParameters());
    setColorParameters(createColorParameters());
    setObjectParameters(createObjectParameters());
    setBooleanParameters(createBooleanParameters());

  }
  
  
  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_TRANSITION_LENGTH, new IntegerParameter(80, 0, 50000));
    
    //3. return the hashtable
    return pParameters;
  }
  
  private Hashtable<String, ColorParameter> createColorParameters() {
    //1. create the hashtable
    Hashtable<String, ColorParameter> pParameters = new Hashtable<String, ColorParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_COLOR_TRANSITION_COLOR, new ColorParameter(new Color(50, 50, 255)));
    
    //3. return the hashtable
    return pParameters;
  
  }
  
  
  
  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    ExampleImageGenerator pGenerator = new ExampleImageGenerator();
    pParameters.put(PARAM_OBJECT_IMAGE, new ObjectParameter(pGenerator.getImage()));
  
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, BooleanParameter> createBooleanParameters() {
    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_BOOL_TRANSITION_TOP, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_TRANSITION_BOTTOM, new BooleanParameter(true));
  
    //3. return the hashtable
    return pParameters;
  }
  
}
  
  
  
