package color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.image.ImageTransformations;

import static color.GrayscaleReductionParameters.*;

/**
 *  Implements the grayscale reduction effect.
 **/
public class GrayscaleReduction extends ImageEffect {

  /*** private member variables ***/
  private boolean m_bRecalculateImage;
  private GrayscaleReductionParameters m_pParameters;
  
  private int m_iGrayscaleNum;
  private BufferedImage m_pGrayscaleImage;
  private BufferedImage m_pImage;

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code GrayscaleReduction} object with the default parameters.
   *
   **/
  public GrayscaleReduction() {
    this(new GrayscaleReductionParameters());
  }
  

  /**
   *   Creates a {@code GrayscaleReduction} object with the specified parameters..
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public GrayscaleReduction(GrayscaleReductionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new GrayscaleReductionParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
    //m_bRecalculateImage = true;
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the grayscale reduction effect.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  
  public void setGrayscaleReductionParameters(GrayscaleReductionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
     
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the grayscale reduction effect. The parameters are only set, 
   *   if the parameters is of the type {@code GrayscaleReductionParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof GrayscaleReductionParameters) {
      setGrayscaleReductionParameters((GrayscaleReductionParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image of the Mandelbrot set according to the parameters that were set.
   *
   *  @return An image of the Mandelbrot set as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    /*//1. check if the parameters have to be reset
    if (m_bRecalculateImage) {
      
    }*/
    
    //3. create a lookup table
    int[] pLookupTable = new int[256];
    double dStepSize = 255/(double)(m_iGrayscaleNum-1);
    for (int i = 0; i < 256; i++) {
      long lMultiplier = Math.round((double)i/dStepSize);
      pLookupTable[i] = (int)(lMultiplier*dStepSize);
    }
    if (!isRunning()) return null;
    
    
    //4. map the image values to the nearest gray scale value
    int iWidth = m_pImage.getWidth();
    int iHeight = m_pImage.getHeight();
    m_pGrayscaleImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    
    int iRGB, iRed, iGreen, iBlue, iGrayValue, iNewGrayValue;
    
    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {
        //fetch the color
        iRGB = m_pImage.getRGB(x,y);
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;
        
        //convert it to gray
        iGrayValue = (int)((double)(iRed + iGreen + iBlue)/3);
        if (iGrayValue > 255) iGrayValue = 255;
        if (iGrayValue < 0) iGrayValue = 0;
        
        //perform the lookup and set the new gray value
        iNewGrayValue = pLookupTable[iGrayValue];
        
        iRGB = (iNewGrayValue << 24) | (iNewGrayValue << 16) | (iNewGrayValue << 8) | iNewGrayValue;

        m_pGrayscaleImage.setRGB(x,y, iRGB);
      }
      if (!isRunning()) return null;
    }
    
    return m_pGrayscaleImage;
    
    
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void fetchParameters() {
    m_iGrayscaleNum = m_pParameters.getIntegerParameterValue(PARAM_INT_GRAYSCALENUM);
    m_pImage = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE);
    
    //2. pass the relevant parameters 
    //   and set the flag if the effect has to be recalculated
    
    //2.1. the integer parameters
    //compareAndSetIntegerParameters(IteratorParameters.PARAM_INT_ITERATION_DEPTH, PARAM_INT_ITERATION_DEPTH);
    
    
    //2.3. the image
    
    
       
    
    
    //4. set the flags
    m_bRecalculateImage = false;
    
    
    
  }
  
  /*private void compareAndSetIntegerParameters(String pIteratorParameterName, String pMandelbrotSetParameterName) {
    int iOldValue = m_pIteratorParameters.getIntegerParameterValue(pIteratorParameterName);
    int iNewValue = m_pParameters.getIntegerParameterValue(pMandelbrotSetParameterName);
    if (iOldValue != iNewValue) {
      m_pIteratorParameters.setIntegerParameterValue(pIteratorParameterName, iNewValue);
      m_bRecalculateRaster = true;
    }
  }*/
  
  
  
  

}
