package color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.util.Random;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;

import static color.NoiseGeneratorParameters.*;

/**
 *  Implements the Noise generator effect.
 **/
public class NoiseGenerator extends ImageEffect {

  /*** private member variables ***/
  private NoiseGeneratorParameters m_pParameters;
  private BufferedImage m_pNoisyImage;
  private BufferedImage m_pImage;
  private boolean m_bCreateSaltAndPepperNoise, m_bCreateGaussianNoise, m_bCreateRandomColorNoise;
  private double m_dSaltAndPepperIntensity, m_dGaussianIntensity, m_dGaussianStandardDeviation, m_dRandomColorIntensity;

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code NoiseGenerator} object with the default parameters.
   *
   **/
  public NoiseGenerator() {
    this(new NoiseGeneratorParameters());
  }
  

  /**
   *   Creates a {@code NoiseGenerator} object with the specified parameters.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public NoiseGenerator(NoiseGeneratorParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new NoiseGeneratorParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the noise generator effect.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public void setNoiseGeneratorParameters(NoiseGeneratorParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
     
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the noise generator effect. The parameters are only set, 
   *   if the object is of the type {@code NoiseGeneratorParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof NoiseGeneratorParameters) {
      setNoiseGeneratorParameters((NoiseGeneratorParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *   Returns the parameters that are currently used for the noise generator effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image processed with the current noise generator settings.
   *
   *  @return An image processed with the current noise generator settings.
   *
   */
  public BufferedImage process() throws Exception {
   
    
    
    
    //1. copy the image 
    int iWidth = m_pImage.getWidth();
    int iHeight = m_pImage.getHeight();
    m_pNoisyImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = m_pNoisyImage.createGraphics();
    pGraphics.drawImage(m_pImage, 0, 0, null);
    
    
    //2. add the noise
    Random pRandom = new Random();
    //2.1. gaussian noise
    if (m_bCreateGaussianNoise) {
      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {
          if (pRandom.nextDouble() < m_dGaussianIntensity) {
          

            Color pCurrentColor = new Color(m_pNoisyImage.getRGB(x,y));
            int iNewRed = pCurrentColor.getRed()+(int)(pRandom.nextGaussian()*m_dGaussianStandardDeviation);
            int iNewGreen = pCurrentColor.getGreen()+(int)(pRandom.nextGaussian()*m_dGaussianStandardDeviation);
            int iNewBlue = pCurrentColor.getBlue()+(int)(pRandom.nextGaussian()*m_dGaussianStandardDeviation);

            if (iNewRed < 0) iNewRed = 0;
            if (iNewRed > 255) iNewRed = 255;
            if (iNewGreen < 0) iNewGreen = 0;
            if (iNewGreen > 255) iNewGreen = 255;
            if (iNewBlue < 0) iNewBlue = 0;
            if (iNewBlue > 255) iNewBlue = 255;

            m_pNoisyImage.setRGB(x,y,(new Color(iNewRed, iNewGreen, iNewBlue)).getRGB());
          
          }
        }
      }
    }
    
    //2.2. salt and pepper
    if (m_bCreateSaltAndPepperNoise) {
      int iWhite = Color.WHITE.getRGB();
      int iBlack = Color.BLACK.getRGB();
      
      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {
          
          if (pRandom.nextDouble() < m_dSaltAndPepperIntensity) {
            if (pRandom.nextBoolean()) {
              m_pNoisyImage.setRGB(x, y, iWhite);
            }
            else {
              m_pNoisyImage.setRGB(x, y, iBlack);
            }
          }
        }
      }
    }
    
    //2.3. random color noise
    if (m_bCreateRandomColorNoise) {
      
      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {
          
          if (pRandom.nextDouble() < m_dRandomColorIntensity) {
            Color pColor = new Color(pRandom.nextInt(256), pRandom.nextInt(256), pRandom.nextInt(256));
            m_pNoisyImage.setRGB(x, y, pColor.getRGB());
          }
        }
      }
    }
    
    
   

    
    //3. return the image
    return m_pNoisyImage;
    
    
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void fetchParameters() {
    
    m_pImage = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE);
    m_bCreateSaltAndPepperNoise = m_pParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE);
    m_bCreateGaussianNoise = m_pParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE);
    m_bCreateRandomColorNoise = m_pParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE);
    
    m_dSaltAndPepperIntensity = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_SALT_AND_PEPPER_INTENSITY);
    m_dGaussianIntensity = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_INTENSITY);
    m_dGaussianStandardDeviation = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_STANDARD_DEVIATION);
    m_dRandomColorIntensity = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_RANDOM_COLOR_INTENSITY);
    
    
    
    
    
  }
  
 
  
  
  
  

}
