package gui;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.*;
import java.awt.Dimension;
import java.awt.image.BufferedImage;


//library imports
import standard.ImageEffect;
import applications.LanguageProperties;
import applications.Settings;
import applications.SettingsDialog;
import applications.ISettingsListener;

import static applications.GuiLanguageProperties.*;

/**
 *  Implementation of an internal frame that shows a properties panel which steers the properties of an image or video effect. 
 **/
public class PropertiesInternalFrame extends JInternalFrame implements ISettingsListener {

  /*** member variables **/
  private AbstractPropertiesPanel m_pPropertiesPanel;
  
  

  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /** 
   *  Creates a {@code PropertiesInternalFrame} with the specified width and height.
   *
   *  @param iWidth The width of the frame.
   *  @param iHeight The height of the frame.
   **/
  public PropertiesInternalFrame(int iWidth, int iHeight) {
    //1. init the frame
    super("", true, true, true, true);
    
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setTitle(pLanguageProperties.getProperty(KEY_PROPERTY_FRAME_TITLE));
    
    if (iWidth < 100) iWidth = 100;
    if (iHeight < 100) iHeight = 100;
    setSize(new Dimension(iWidth,iHeight));
    setClosable(false);
    
    //2. init the members
    m_pPropertiesPanel = null;
    
    
  }
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the properties panel that should be shown by this internal frame. 
   *
   *  @param pPropertiesPanel The {@code AbstractPropertiesPanel} object that should be shown by this internal frame. 
   **/
  public void setPropertiesPanel(AbstractPropertiesPanel pPropertiesPanel) {
    //1. make a simple check
    if (pPropertiesPanel == null) return;
    
    //2. remove the old panel
    if (m_pPropertiesPanel != null) {
      remove(m_pPropertiesPanel);
    }
    
    //3. set the new panel
    m_pPropertiesPanel = pPropertiesPanel;
    add(m_pPropertiesPanel);
    repaint(); //is needed to show the new panel immediately.
    
    
      
  }
  
  
  ////////////////////////////////////
  // ISettingsListener METHODS      //
  ////////////////////////////////////
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent and the properties panel can adapt accordingly.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public void applyNewSettings(Settings pSettings) {
    if (pSettings == null) return;
    
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setTitle(pLanguageProperties.getProperty(KEY_PROPERTY_FRAME_TITLE));
    
    m_pPropertiesPanel.applyNewSettings(pSettings);
  }
  
 
  
  
  ////////////////////////////////
  // PRIVATE HELPER METHODS     //
  ////////////////////////////////

}

