package gui.color;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import color.GrayscaleReduction;
import color.GrayscaleReductionParameters;
import gui.AbstractPropertiesPanel;
import gui.ISliderPanelListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;


import gui.image.ImageLoadPanel;
import gui.image.IImageLoadPanelListener;

import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static color.GrayscaleReductionParameters.*;
import static applications.GrayscaleReductionLanguageProperties.*;

/**
 *  A properties panel for the gray scale reduction effect.
 **/
public class GrayscaleReductionPropertiesPanel
  extends AbstractPropertiesPanel
  implements ISliderPanelListener, IImageLoadPanelListener {

  /*** member variables ***/
  private GrayscaleReductionParameters m_pParameters;

  /*** gui member variables ***/
  //private FileLoadPanel m_pFileLoadPanel;
  private ImageLoadPanel m_pImageLoadPanel;
  private SliderPanel m_pGrayscaleNumSliderPanel;


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code GrayscaleReductionPropertiesPanel} object with the default parameters
   *  of the {@code GrayscaleReduction} effect.
   **/
  public GrayscaleReductionPropertiesPanel() {
    //1. init members
    m_pParameters = null;



    //4. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addPanelListener(this);


    //5. create the panel for the grayscale num
    m_pGrayscaleNumSliderPanel = new IntegerSliderPanel(1, 64, 8);
    m_pGrayscaleNumSliderPanel.addPanelListener(this);

    initLayout();

  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code GrayScaleReductionParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code GrayScaleReductionParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }

  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code GrayscaleReductionPropertiesPanel} object.
   *  The set operation is only successful if a {@code GrayscaleReductionParameters} object has been specified.
   *
   *  @param pParameters The {@code GrayscaleReductionParameters} object representing the effect parameters
   *                     that should be represented by this {@code GrayscaleReductionPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof GrayscaleReductionParameters) {
      m_pParameters = (GrayscaleReductionParameters)pParameters;

      updateControls();
    }
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pImageLoadPanel);
    pPanels.add(m_pGrayscaleNumSliderPanel);


    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_IMAGE);
    pKeys.add(KEY_GRAYSCALE_NUM);


    return pKeys;
  }


  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the number of gray scales has been changed in the slider panel for this parameter, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that steers the number of gray scales.
   *  @param iValue The number of gray scales.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pParameters == null) m_pParameters = new GrayscaleReductionParameters();

    //2. set the new parameters
    m_pParameters.setIntegerParameterValue(PARAM_INT_GRAYSCALENUM, iValue);

    //3. fire the apply changes event
    fireApplyChanges(m_pParameters);

  }



  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image to which the effect should be applied is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image to which the effect is applied.
   *  @param pImage The image to which the effect should be applied.
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pParameters == null) m_pParameters = new GrayscaleReductionParameters();

    //1. set the new colors
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);

    //2. fire the apply changes event
    fireApplyChanges(m_pParameters);
  }


  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  protected void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();




    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }




  }


  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
  }


}
