package gui.color;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JLabel;

import java.awt.Color;
import java.awt.FlowLayout;

import java.util.Vector;

//library imports
import gui.ISliderPanelListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.JPanelWithListener;

/**
 *  A panel representing a transparent color that can be modified with a transparency slider and a {@code ColorDisplay} object. 
 **/
public class TransparentColorPanel extends JPanelWithListener<ITransparentColorPanelListener> 
  implements ISliderPanelListener, IColorDisplayListener {
  
  /** private gui member variables **/
  private ColorDisplay m_pColorDisplay;
  private SliderPanel m_pTransparencySliderPanel;
  
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates a default {@code TransparentColorPanel} object with a the color black and the transparency 0.
   **/
  public TransparentColorPanel() {
  
    setLayout(new FlowLayout());
      
    m_pColorDisplay = new ColorDisplay();
    m_pColorDisplay.setColor(Color.BLACK);
    m_pColorDisplay.addPanelListener(this);
    add(m_pColorDisplay);
    
    add(new JLabel("Transparency"));
    
    m_pTransparencySliderPanel = new IntegerSliderPanel(0, 100, 0);
    m_pTransparencySliderPanel.addPanelListener(this);
    add(m_pTransparencySliderPanel);
    
    
    
  
  }
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *  Returns the color that is represented by this {@code TransparentColorPanel} object.
   *
   *  @return The color that is represented by this {@code TransparentColorPanel} object.
   */
  public Color getColor() {
    return calculateColor();
  }
  
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the title of the color chooser window of the {@code ColorDisplay} object.
   *
   *  @param pText The title of the color chooser window.
   **/
  public void setText(String pText) {
    m_pColorDisplay.setColorChooserTitle(pText);
  }
  
  /**
   *  Sets the color that should be represented by this {@code TransparentColorPanel} object.
   *
   *  @param pColor The color that should be represented by this {@code TransparentColorPanel} object.
   */
  public void setColor(Color pColor) {
    Color pDisplayColor = new Color(pColor.getRed(), pColor.getGreen(), pColor.getBlue());
    m_pColorDisplay.setColor(pDisplayColor);
    
    int iAlpha = (int)(pColor.getAlpha()/2.55);
    m_pTransparencySliderPanel.setIntValue(100 - iAlpha);
  }
  

  
  
  ///////////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER                //
  ///////////////////////////////////////////////
  /**
   *  If the transparency slider has been adapted , it is 
   *  communicated with this method, the color is adapted accordingly and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {
    
    applyChanges();
  }
  
  
  ///////////////////////////////////////////////
  // COLOR DISPLAY LISTENER                    //
  ///////////////////////////////////////////////
  /**
   *  If the color has been changed in the {@code ColorDisplay}, this is communicated with this method, 
   *  the transparent color is adapted accordingly and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that steers the color part of the transparent color.
   *  @param pColor The color that has been chosen. 
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    
    applyChanges();
  }
  
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private Color calculateColor() {
    Color pColor = m_pColorDisplay.getColor();
    
    int iRed = pColor.getRed();
    int iGreen = pColor.getGreen();
    int iBlue = pColor.getBlue();
    int iAlpha = 100 - m_pTransparencySliderPanel.getIntValue();
    
    return new Color(iRed/255.0f, iGreen/255.0f, iBlue/255.0f, iAlpha/100.0f);
  }
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  protected void applyChangesToListener(ITransparentColorPanelListener pListener) {
    Color pNewColor = calculateColor();
    pListener.applyNewColor(this, pNewColor);
  }
  
  
  
  
}