package gui.drawing;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import drawing.StarsParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SizingPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;

//static imports
import static drawing.StarsParameters.*;
import static applications.StarsLanguageProperties.*;


/**
 *  A properties panel for the stars effect.
 **/
public class StarsPropertiesPanel extends AbstractPropertiesPanel
  implements ISliderPanelListener, IColorDisplayListener {

  /*** member variables ***/
  private StarsParameters m_pStarsParameters;

  /*** gui member variables ***/
  private SliderPanel m_pPikeNumSliderPanel, m_pPikeLengthSliderPanel, m_pLineNumberSliderPanel;
  private SliderPanel m_pXTranslateSliderPanel, m_pYTranslateSliderPanel, m_pRotationSliderPanel;
  private ColorDisplay m_pLineColorDisplay, m_pBackgroundColorDisplay;
  private SizingPanel m_pSizingPanel;

  private StarsPanel m_pStarsPanel;

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code StarsPropertiesPanel} object with the default parameters
   *  of the {@code Stars} effect.
   **/
  public StarsPropertiesPanel(StarsPanel pStarsPanel) {
    //0. init members
    m_pStarsParameters = null;
    m_pStarsPanel = pStarsPanel;


    //2. create the panel for the pike nums
    m_pPikeNumSliderPanel = new IntegerSliderPanel(3, 20, 4);
    m_pPikeNumSliderPanel.addPanelListener(this);


    //3. create the panel for the pike length
    m_pPikeLengthSliderPanel = new IntegerSliderPanel(20, 1000, 180);
    m_pPikeLengthSliderPanel.addPanelListener(this);

    //3. create the line number panel
    m_pLineNumberSliderPanel = new IntegerSliderPanel(5, 100, 10);
    m_pLineNumberSliderPanel.addPanelListener(this);

    //4. create the line color panel
    m_pLineColorDisplay = new ColorDisplay();
    m_pLineColorDisplay.addPanelListener(this);

    //5. create background color panel
    m_pBackgroundColorDisplay = new ColorDisplay();
    m_pBackgroundColorDisplay.addPanelListener(this);

    //6. create the horizontal translation slider panel
    m_pXTranslateSliderPanel = new IntegerSliderPanel(-1000, 1000, 0);
    m_pXTranslateSliderPanel.addPanelListener(this);

    //7. create the vertical translation slider panel
    m_pYTranslateSliderPanel = new IntegerSliderPanel(-1000, 1000, 0);
    m_pYTranslateSliderPanel.addPanelListener(this);

    //8. create the rotation slider panel
    m_pRotationSliderPanel = new IntegerSliderPanel(0, 360, 0);
    m_pRotationSliderPanel.addPanelListener(this);


    //9. sizing panel
    m_pSizingPanel = new SizingPanel();
    m_pSizingPanel.addPanelListener(m_pStarsPanel);


    initLayout();


  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code StarsParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code StarsParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pStarsParameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pPikeNumSliderPanel);
    pPanels.add(m_pPikeLengthSliderPanel);
    pPanels.add(m_pLineNumberSliderPanel);
    pPanels.add(m_pLineColorDisplay);
    pPanels.add(m_pBackgroundColorDisplay);
    pPanels.add(m_pXTranslateSliderPanel);
    pPanels.add(m_pYTranslateSliderPanel);
    pPanels.add(m_pRotationSliderPanel);
    pPanels.add(m_pSizingPanel);

    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_PIKE_NUM);
    pKeys.add(KEY_PIKE_LENGTH);
    pKeys.add(KEY_LINE_NUM);
    pKeys.add(KEY_LINE_COLOR);
    pKeys.add(KEY_BACKGROUND_COLOR);
    pKeys.add(KEY_TRANSLATION_X);
    pKeys.add(KEY_TRANSLATION_Y);
    pKeys.add(KEY_ROTATION);
    pKeys.add(KEY_SIZING);

    return pKeys;
  }


  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code StarsPropertiesPanel} object.
   *  The set operation is only successful if a {@code StarsParameters} object has been specified.
   *
   *  @param pParameters The {@code StarsParameters} object representing the effect parameters
   *                     that should be represented by this {@code StarsPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof StarsParameters) {
      m_pStarsParameters = (StarsParameters)pParameters;

      updateControls();
    }
  }




  ///////////////////////////////////////
  // SLIDER PANEL LISTENER             //
  ///////////////////////////////////////
  /**
   *  If the number of spikes, the spike length, the line number, the translation or rotation values have been adapted
   *  in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pStarsParameters == null) m_pStarsParameters = new StarsParameters();

    //1. set the new parameters
    if (pSliderPanel.equals(m_pPikeNumSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_SPIKE_NUM, iValue);
    }


    if (pSliderPanel.equals(m_pPikeLengthSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_SPIKE_LENGTH, iValue);
    }

    if (pSliderPanel.equals(m_pLineNumberSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_LINE_NUM, iValue);
    }

    if (pSliderPanel.equals(m_pXTranslateSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_HORIZONTAL_TRANSLATION, iValue);
    }

    if (pSliderPanel.equals(m_pYTranslateSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_VERTICAL_TRANSLATION, iValue);
    }

    if (pSliderPanel.equals(m_pRotationSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_ROTATION, iValue);
    }


    //2. fire the apply changes event
    fireApplyChanges(m_pStarsParameters);
  }


  /////////////////////////////////////////
  // ICOLOR PANEL LISTENER               //
  /////////////////////////////////////////
  /**
   *  If the color of the lines has been changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that has been changed.
   *  @param pColor The color that has been set for the {@code ColorDisplay} object.
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pStarsParameters == null) m_pStarsParameters = new StarsParameters();

    //1. set the new color
    if (pColorDisplay.equals(m_pLineColorDisplay)) {
      m_pStarsParameters.setColorParameterValue(PARAM_COLOR_LINE_COLOR, pColor);
    }
    else {
      m_pStarsParameters.setColorParameterValue(PARAM_COLOR_BACKGROUND, pColor);
    }



    //2. fire the apply changes event
    fireApplyChanges(m_pStarsParameters);
  }





  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pStarsParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    m_pLineColorDisplay.setColor(m_pStarsParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR));
    m_pBackgroundColorDisplay.setColor(m_pStarsParameters.getColorParameterValue(PARAM_COLOR_BACKGROUND));
  }











}
