package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Color;
import java.awt.image.BufferedImage;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.util.Vector;

//library imports

import fractals.JuliaSetParameters;
import fractals.MandelbrotSetParameters;
import fractals.JuliaSetWithMandelbrotSet;
import gui.AbstractPropertiesPanel;
import gui.AbstractInteractionImageEffectPanel;
import util.color.ColorGradient;
import standard.EffectParameters;
import util.math.ComplexNumber;
import gui.ImageEffectSaveOption;

import static fractals.FractalSetParameters.*;
import static applications.FractalSetLanguageProperties.*;



/**
 *  Handles the interaction with a Mandelbrot set.
 **/

public class JuliaSetWithMandelbrotPanel extends AbstractInteractionImageEffectPanel implements MouseListener, MouseMotionListener {


  //effect members
  private JuliaSetWithMandelbrotPropertiesPanel m_pPropertiesPanel;
  private JuliaSetParameters m_pJuliaParams;
  private MandelbrotSetParameters m_pMBParams;
  private int m_iWidth, m_iHeight;
  private JuliaSetWithMandelbrotSet m_pEffect;

  //interaction members
  private Point m_pStartPoint, m_pEndPoint;

  //private constants
  private static final int CROSS_SIZE = 10;


  ////////////////////////////////////
  // CONSTRUCTOR                    //
  ////////////////////////////////////
  /**
   *  Creates a {@code MandelbrotSetPanel} object with the default settings for the Mandelbrot set effect.
   **/
  public JuliaSetWithMandelbrotPanel() {

    m_pPropertiesPanel = null;
    m_pJuliaParams = new JuliaSetParameters();
    m_pMBParams = new MandelbrotSetParameters();
    m_pEffect = new JuliaSetWithMandelbrotSet(m_pJuliaParams, m_pMBParams);
    m_iWidth = 0;
    m_iHeight = 0;

    m_pStartPoint = null;
    m_pEndPoint = null;

    addMouseListener(this);
    addMouseMotionListener(this);
  }

  ///////////////////////////
  // PROTECTED METHODS     //
  ///////////////////////////
  /**
   *  Draws the interaction graphics on top of the result image.
   *
   *  @param pGraphics The graphics context that is used to perform the drawings.
   **/
  protected void drawInteraction(Graphics2D pGraphics) {
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {
      pGraphics.setColor(Color.WHITE);
      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)(m_iWidth/2);
      int iYSize = (int)(iXSize*dAspectRatio);
      if (iXSize > 0) {
        pGraphics.drawRect(m_pStartPoint.x, m_pStartPoint.y, iXSize, iYSize);
      }
      else {
        pGraphics.drawRect(m_pStartPoint.x+iXSize, m_pStartPoint.y+iYSize, -iXSize, -iYSize);
      }

    }

    Point pCPoint = m_pEffect.getNearesPixelOfParameterC();
    if (pCPoint != null) {
      pGraphics.setColor(Color.RED);
      pGraphics.drawLine(pCPoint.x-CROSS_SIZE, pCPoint.y, pCPoint.x+CROSS_SIZE, pCPoint.y);
      pGraphics.drawLine(pCPoint.x, pCPoint.y-CROSS_SIZE, pCPoint.x, pCPoint.y+CROSS_SIZE);
    }
  }

  protected void setEffectSize(int iWidth, int iHeight) {
    if ((iWidth != m_iWidth) || (iHeight != m_iHeight)) {

      m_pJuliaParams.setEffectSize(iWidth/2, iHeight);
      if (m_iWidth == 0) m_pJuliaParams.setDefaultJuliaSet(); //julia set does not have the correct section at the beginning


      m_pMBParams.setEffectSize(iWidth/2, iHeight);


      m_iWidth = iWidth;
      m_iHeight = iHeight;

      parametersChanged();
    }
  }


  /////////////////////////
  // GET METHODS         //
  /////////////////////////
  /**
   *   Returns the parameters that are currently used by the effect that is shown in this panel.
   *
   *   @return The parameters that are currently used for the effect that is shown in this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pJuliaParams;
  }

  public EffectParameters getJuliaSetParameters() {
    return m_pJuliaParams;
  }

  public EffectParameters getMandelbrotSetParameters() {
    return m_pMBParams;
  }

  /**
   *  Returns the {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   *
   *  @return The {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   **/
  public AbstractPropertiesPanel getPropertiesPanel() {
    if (m_pPropertiesPanel == null) {
      m_pPropertiesPanel = new JuliaSetWithMandelbrotPropertiesPanel(this);
    }

    return m_pPropertiesPanel;
  }

  public Vector<ImageEffectSaveOption> getSaveOptions() {
    Vector<ImageEffectSaveOption> saveOptions = new Vector<ImageEffectSaveOption>();
    saveOptions.add(new ImageEffectSaveOption(KEY_SAVE_MANDELBROT_SET, false));
    saveOptions.add(new ImageEffectSaveOption(KEY_SAVE_JULIA_SET, true));
    return saveOptions;
  }

  public BufferedImage getEffectImageForSaving(Vector<ImageEffectSaveOption> saveOptions) throws Exception {
    boolean bSaveJulia = true;
    boolean bSaveMandelbrot = false;
    if (saveOptions != null) {
      for (int i = 0; i < saveOptions.size(); i++) {
        ImageEffectSaveOption saveOption = saveOptions.get(i);
        if (saveOption.getLabelKey().equals(KEY_SAVE_MANDELBROT_SET)) {
          bSaveMandelbrot = saveOption.isSelected();
        }
        else if (saveOption.getLabelKey().equals(KEY_SAVE_JULIA_SET)) {
          bSaveJulia = saveOption.isSelected();
        }
      }
    }

    if (bSaveJulia && bSaveMandelbrot) {
      return getEffectImage();
    }
    else if (bSaveJulia) {
      return m_pEffect.getJuliaImage();
    }
    else if (bSaveMandelbrot) {
      return m_pEffect.getMandelbrotImage();
    }
    else {
      throw new Exception(KEY_SAVE_JULIA_MB_EXCEPTION);
    }


  }

  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  public void setCParameter(double dReal, double dImaginary) {
    m_pJuliaParams.setCParameter(dReal, dImaginary);
    parametersChanged(true, false);
  }

  /**
   *   Sets the parameters for the image effect that is shown in the panel. The parameters are only set,
   *   if the object is of the correct parameter type.
   *
   *   @param pEffectParameters The parameters that configure the image effect.
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    //TODO
  }

  public void setDefaultJuliaSet() {
    m_pJuliaParams.setDefaultFractalSet();
    parametersChanged(true, false);
  }

  public void setDefaultMandelbrotSet() {
    m_pMBParams.setDefaultFractalSet();
    parametersChanged(false, true);
  }

  public void setIterationDepth(boolean bJulia, int iIterationDepth) {
    if (bJulia) {
      m_pJuliaParams.setIntegerParameterValue(PARAM_INT_ITERATION_DEPTH, iIterationDepth);
    }
    else {
      m_pMBParams.setIntegerParameterValue(PARAM_INT_ITERATION_DEPTH, iIterationDepth);
    }
    parametersChanged(bJulia, !bJulia);
  }

  public void setColorGradient(boolean bJulia, ColorGradient colorGradient) {
    if (bJulia) {
      m_pJuliaParams.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, colorGradient);
    }
    else {
      m_pMBParams.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, colorGradient);
    }
    parametersChanged(bJulia, !bJulia);
  }

  public void setJuliaWindowCorners(double dUpperLeftX, double dUpperLeftY, double dLowerRightX, double dLowerRightY) {

    m_pJuliaParams.setWindowCorners(new ComplexNumber(dUpperLeftX, dUpperLeftY), new ComplexNumber(dLowerRightX, dLowerRightY));
    juliaParametersChanged();
  }

  public void setMandelbrotWindowCorners(double dUpperLeftX, double dUpperLeftY, double dLowerRightX, double dLowerRightY) {

    m_pMBParams.setWindowCorners(new ComplexNumber(dUpperLeftX, dUpperLeftY), new ComplexNumber(dLowerRightX, dLowerRightY));
    mandelbrotParametersChanged();
  }

  public void moveDownJulia(double dStepSize) {
    m_pJuliaParams.moveDown(dStepSize);
    juliaParametersChanged();
  }

  public void moveUpJulia(double dStepSize) {
    m_pJuliaParams.moveUp(dStepSize);
    juliaParametersChanged();
  }

  public void moveLeftJulia(double dStepSize) {
    m_pJuliaParams.moveLeft(dStepSize);
    juliaParametersChanged();
  }

  public void moveRightJulia(double dStepSize) {
    m_pJuliaParams.moveRight(dStepSize);
    juliaParametersChanged();
  }


  public void moveDownMandelbrot(double dStepSize) {
    m_pMBParams.moveDown(dStepSize);
    mandelbrotParametersChanged();
  }

  public void moveUpMandelbrot(double dStepSize) {
    m_pMBParams.moveUp(dStepSize);
    mandelbrotParametersChanged();
  }

  public void moveLeftMandelbrot(double dStepSize) {
    m_pMBParams.moveLeft(dStepSize);
    mandelbrotParametersChanged();
  }

  public void moveRightMandelbrot(double dStepSize) {
    m_pMBParams.moveRight(dStepSize);
    mandelbrotParametersChanged();
  }


  /////////////////////////////////
  // MOUSE LISTENER INTERFACE    //
  /////////////////////////////////
  /**
   *  If the mouse is double clicked the default Mandelbrot set is shown again.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseClicked(MouseEvent e) {
    //doubleclick reset to default
    if (e.getClickCount() == 2) {
      if (m_pEffect.isJuliaSet(e.getX())) {
        setDefaultJuliaSet();
      }
      else {
        setDefaultMandelbrotSet();
      }
    }
    else {
      if (!m_pEffect.isJuliaSet(e.getX())) {
        ComplexNumber pCParameter = m_pEffect.getComplexNumberAt(e.getX(), e.getY());
        setCParameter(pCParameter.m_dReal, pCParameter.m_dImaginary);
      }
    }
  }

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseEntered(MouseEvent e) {}

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseExited(MouseEvent e) {}

  /**
   *  Sets a start point for a zoom operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mousePressed(MouseEvent e) {
    m_pStartPoint = new Point(e.getX(), e.getY());
  }

  /**
   *  Realizes the zooming.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseReleased(MouseEvent e) {
    //set the new coordinates of the mandelbrot set
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {


      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)(m_iWidth/2);
      int iYSize = (int)(iXSize*dAspectRatio);

      int iXPos1, iXPos2, iYPos1, iYPos2;
      if (iXSize > 0) {
        iXPos1 = m_pStartPoint.x;
        iYPos1 = m_pStartPoint.y;

        iXPos2 = iXPos1 + iXSize;
        iYPos2 = iYPos1 + iYSize;

      }
      else {
        iXPos1 = m_pStartPoint.x + iXSize;
        iYPos1 = m_pStartPoint.y + iYSize;

        iXPos2 = m_pStartPoint.x;
        iYPos2 = m_pStartPoint.y;

      }

      //correct the iXPos and the iYPos by the image offset
      int iXOffset = getImageXOffset();
      iXPos1 -= iXOffset;
      iXPos2 -= iXOffset;

      int iYOffset = getImageYOffset();
      iYPos1 -= iYOffset;
      iYPos2 -= iYOffset;



      ComplexNumber pUpperLeft = m_pEffect.getComplexNumberAt(iXPos1, iYPos1);
      ComplexNumber pLowerRight = m_pEffect.getComplexNumberAt(iXPos2, iYPos2);

      if (m_pEffect.isJuliaSet(iXPos1)) {
        m_pJuliaParams.setWindowCorners(pUpperLeft, pLowerRight);
        parametersChanged(true, false);
      }
      else {
        m_pMBParams.setWindowCorners(pUpperLeft, pLowerRight);
        parametersChanged(false, true);
      }
    }




    //reset the interaction variables
    m_pStartPoint = null;
    m_pEndPoint = null;

    //repaint
    repaint();
  }



  /////////////////////////////////////
  // MOUSE MOTION LISTENER INTERFACE //
  /////////////////////////////////////
  /**
   *  Sets an end point for the zooming operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseDragged(MouseEvent e) {
    if (m_pStartPoint != null) {
      m_pEndPoint = new Point(e.getX(), e.getY());
      repaint();
    }
  }

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseMoved(MouseEvent e) {}



  //////////////////////////
  // PRIVATE METHODS      //
  //////////////////////////
  private void parametersChanged() {
    parametersChanged(true, true);
  }

  private void juliaParametersChanged() {
    parametersChanged(true, false);
  }

  private void mandelbrotParametersChanged() {
    parametersChanged(false, true);
  }

  private void parametersChanged(boolean bJuliaChanged, boolean bMBChanged) {
    applyChanges();
    initEffectRepaint(bJuliaChanged, bMBChanged);
  }

  private void initEffectRepaint(boolean bJuliaChanged, boolean bMBChanged) {
    if (m_pEffect == null) return;

    if (bJuliaChanged) m_pEffect.setJuliaSetParameters(m_pJuliaParams);
    if (bMBChanged) m_pEffect.setMandelbrotSetParameters(m_pMBParams);
    m_pEffect.processAsThread(this);
  }









}
