package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JComboBox;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.util.Vector;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;


//library imports
import standard.EffectParameters;
import gui.AbstractPropertiesPanel;
import gui.IntegerSliderPanel;
import gui.SliderPanel;
import gui.ISliderPanelListener;
import gui.IImageEffectPanelListener;
import gui.AbstractImageEffectPanel;
import gui.color.ColorGradientPanel;
import gui.color.IColorGradientPanelListener;
import util.color.ColorGradient;
import util.math.ComplexNumber;
import fractals.MandelbrotSetParameters;
import fractals.JuliaSetParameters;

import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

import static applications.FractalSetLanguageProperties.*;


public class JuliaSetWithMandelbrotPropertiesPanel extends AbstractPropertiesPanel implements
  ISliderPanelListener, IImageEffectPanelListener, IColorGradientPanelListener,
  IFractalSetNavigationPanelListener, IFractalSetButtonNavigationPanelListener,
  IJuliaSetParameterCPanelListener

  {


  /*** member variables ***/
  private JuliaSetWithMandelbrotPanel m_pEffectPanel;
  private IntegerSliderPanel m_pJuliaIterationDepth, m_pMBIterationDepth;
  private ColorGradientPanel m_pJuliaColorGradient, m_pMBColorGradient;
  private FractalSetNavigationPanel m_pJuliaNavigationPanel, m_pMBNavigationPanel;
  private FractalSetButtonNavigationPanel m_pJuliaButtonNavigationPanel, m_pMBButtonNavigationPanel;
  private JuliaSetParameterCPanel m_pParameterCPanel;

  ///////////////////////////////////
  // CONSTRUCTOR                   //
  ///////////////////////////////////
  public JuliaSetWithMandelbrotPropertiesPanel(JuliaSetWithMandelbrotPanel pEffectPanel) {
    m_pEffectPanel = pEffectPanel;
    m_pEffectPanel.addPanelListener(this);


    //2. iteration depths
    m_pJuliaIterationDepth = new IntegerSliderPanel(10, 1000, 100);
    m_pJuliaIterationDepth.addPanelListener(this);

    m_pMBIterationDepth = new IntegerSliderPanel(10, 1000, 100);
    m_pMBIterationDepth.addPanelListener(this);


    //3. color GRADIENT
    m_pJuliaColorGradient = new ColorGradientPanel();
    m_pJuliaColorGradient.addPanelListener(this);
    m_pJuliaColorGradient.setMinimumSize(new Dimension(400, 50));
    m_pJuliaColorGradient.setPreferredSize(new Dimension(400, 50));

    m_pMBColorGradient = new ColorGradientPanel();
    m_pMBColorGradient.addPanelListener(this);
    m_pMBColorGradient.setMinimumSize(new Dimension(400, 50));
    m_pMBColorGradient.setPreferredSize(new Dimension(400, 50));

    //4. navigation
    m_pJuliaNavigationPanel = new FractalSetNavigationPanel();
    setEffectParameters(m_pEffectPanel.getJuliaSetParameters());
    m_pJuliaNavigationPanel.addPanelListener(this);

    m_pMBNavigationPanel = new FractalSetNavigationPanel();
    setEffectParameters(m_pEffectPanel.getMandelbrotSetParameters());
    m_pMBNavigationPanel.addPanelListener(this);

    m_pJuliaButtonNavigationPanel = new FractalSetButtonNavigationPanel();
    m_pJuliaButtonNavigationPanel.addPanelListener(this);

    m_pMBButtonNavigationPanel = new FractalSetButtonNavigationPanel();
    m_pMBButtonNavigationPanel.addPanelListener(this);

    //parameter c
    m_pParameterCPanel = new JuliaSetParameterCPanel();
    m_pParameterCPanel.addPanelListener(this);

    initLayout();



  }

  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    if (m_pEffectPanel == null) return null;
    return m_pEffectPanel.getEffectParameters();
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pMBIterationDepth);
    pPanels.add(m_pMBColorGradient);
    pPanels.add(m_pMBNavigationPanel);
    pPanels.add(m_pMBButtonNavigationPanel);
    pPanels.add(m_pJuliaIterationDepth);
    pPanels.add(m_pJuliaColorGradient);
    pPanels.add(m_pJuliaNavigationPanel);
    pPanels.add(m_pJuliaButtonNavigationPanel);
    pPanels.add(m_pParameterCPanel);




    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_MANDELBROT_SET + ";: ;" + KEY_ITERATION_DEPTH);
    pKeys.add(KEY_MANDELBROT_SET + ";: ;" + KEY_COLOR_GRADIENT);
    pKeys.add(KEY_MANDELBROT_SET + ";: ;" + KEY_NAVIGATION);
    pKeys.add(KEY_MANDELBROT_SET + ";: ;" + KEY_NAVIGATION2);
    pKeys.add(KEY_JULIA_SET + ";: ;" + KEY_ITERATION_DEPTH);
    pKeys.add(KEY_JULIA_SET + ";: ;" + KEY_COLOR_GRADIENT);
    pKeys.add(KEY_JULIA_SET + ";: ;" + KEY_NAVIGATION);
    pKeys.add(KEY_JULIA_SET + ";: ;" + KEY_NAVIGATION2);
    pKeys.add(KEY_JULIA_SET + ";: ;" + KEY_PARAMETER_C);

    //pPropertyKeys.add(KEY_SIZE);


    return pKeys;
  }


  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the effect parameters
   *                     that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof JuliaSetParameters) {
      JuliaSetParameters params = (JuliaSetParameters)pParameters;

      ComplexNumber pUpperLeftCorner = params.getUpperLeftWindowCorner();
      ComplexNumber pRightLowerCorner = params.getLowerRightWindowCorner();
      m_pJuliaNavigationPanel.setWindowCorners(pUpperLeftCorner.m_dReal, pUpperLeftCorner.m_dImaginary,
        pRightLowerCorner.m_dReal, pRightLowerCorner.m_dImaginary);

      if (m_pParameterCPanel != null) {
        ComplexNumber pParameterC = params.getCParameter();
        m_pParameterCPanel.setParameterC(pParameterC.m_dReal, pParameterC.m_dImaginary);
      }


    }
    else if (pParameters instanceof MandelbrotSetParameters) {
      MandelbrotSetParameters params = (MandelbrotSetParameters)pParameters;

      ComplexNumber pUpperLeftCorner = params.getUpperLeftWindowCorner();
      ComplexNumber pRightLowerCorner = params.getLowerRightWindowCorner();
      m_pMBNavigationPanel.setWindowCorners(pUpperLeftCorner.m_dReal, pUpperLeftCorner.m_dImaginary,
        pRightLowerCorner.m_dReal, pRightLowerCorner.m_dImaginary);
    }
  }

  //////////////////////////////////////////////
  // PROTECTED LISTENER METHODS FOR INSTANCES //
  //////////////////////////////////////////////
  /**
   *  Informs all registered listeners that a change in the properties panel has been made. The image effect parameters
   *  of this properties panel are passed on to the listeners.
   *
   *  @param pEffectParameters The image effect parameters that should be passed on to the listeners.
   *
   **/
  protected void fireApplyChanges(EffectParameters pEffectParameters) {
    super.fireApplyChanges(pEffectParameters);

    setEffectParameters(pEffectParameters);
  }


  ///////////////////////////////////
  // IMAGE EFFECT PANEL LISTENER   //
  ///////////////////////////////////
  public void applyImageEffectPanelChanges(AbstractImageEffectPanel pPanel) {
    setEffectParameters(pPanel.getEffectParameters());
    if (pPanel instanceof JuliaSetWithMandelbrotPanel) {
      setEffectParameters(((JuliaSetWithMandelbrotPanel)pPanel).getMandelbrotSetParameters());
    }
  }



  ///////////////////////////////////
  // SLIDER PANEL LISTENER         //
  ///////////////////////////////////
  /**
   *  If the iteration depth has been adapted
   *  in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pEffectPanel == null) return;

    //2. set the new parameters
    boolean bJulia = pSliderPanel.equals(m_pJuliaIterationDepth);
    m_pEffectPanel.setIterationDepth(bJulia, iValue);



  }

  ////////////////////////////////////////
  // COLOR GRADIENT PANEL LISTENER      //
  ////////////////////////////////////////
  /**
   *  If the color gradient has been modified in the specified {@code ColorGradientPanel}, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorGradientPanel The {@code ColorGradientPanel} object that has been modified.
   **/
  public void colorGradientChanged(ColorGradientPanel pColorGradientPanel) {
    //1. make some checks
    if (pColorGradientPanel == null) return;
    if (m_pEffectPanel == null) return;

    //2. set the color gradient
    ColorGradient pColorGradient = pColorGradientPanel.getColorGradient();
    if (pColorGradient == null) return;

    boolean bJulia = pColorGradientPanel.equals(m_pJuliaColorGradient);
    m_pEffectPanel.setColorGradient(bJulia, pColorGradient);



  }

  ////////////////////////////////////////
  // COMPONENT LISTENER                 //
  ////////////////////////////////////////
  protected void componentResizedForChildren(ComponentEvent e, JPanel currentPropertyPanel) {
    handleColorGradientPanelSet(currentPropertyPanel);
  }

  //////////////////////////////////
  // PROPERTY PANEL CHANGED       //
  //////////////////////////////////
  protected void propertyPanelChanged(JPanel currentPropertyPanel) {
    handleColorGradientPanelSet(currentPropertyPanel);
  }

  /////////////////////////////////////
  // NAVIGATION PANEL LISTENER       //
  /////////////////////////////////////
  public void applyResetFractalSet(FractalSetNavigationPanel panel) {
    //m_bChangeListenerInAction = true;
    if (panel.equals(m_pJuliaNavigationPanel)) {
      m_pEffectPanel.setDefaultJuliaSet();
    }
    else {
      m_pEffectPanel.setDefaultMandelbrotSet();
    }
    //m_bChangeListenerInAction = false;
  }

  public void applyCornerChange(FractalSetNavigationPanel panel,
    double dLeftUpperX, double dLeftUpperY, double dRightLowerX, double dRightLowerY) {

    //m_bChangeListenerInAction = true;
    if (panel.equals(m_pJuliaNavigationPanel)) {
      m_pEffectPanel.setJuliaWindowCorners(dLeftUpperX, dLeftUpperY, dRightLowerX, dRightLowerY);
    }
    else {
      m_pEffectPanel.setMandelbrotWindowCorners(dLeftUpperX, dLeftUpperY, dRightLowerX, dRightLowerY);
    }
    //m_bChangeListenerInAction = false;
  }

  /////////////////////////////////////////
  // BUTTON NAVIGATION PANEL LISTENER   //
  //////////////////////////////////////
  public void moveDown(FractalSetButtonNavigationPanel panel, double dStepSize) {
    if (panel.equals(m_pJuliaButtonNavigationPanel)) {
      m_pEffectPanel.moveDownJulia(dStepSize);
    }
    else {
      m_pEffectPanel.moveDownMandelbrot(dStepSize);
    }
  }

  public void moveUp(FractalSetButtonNavigationPanel panel, double dStepSize) {
    if (panel.equals(m_pJuliaButtonNavigationPanel)) {
      m_pEffectPanel.moveUpJulia(dStepSize);
    }
    else {
      m_pEffectPanel.moveUpMandelbrot(dStepSize);
    }
  }
  public void moveLeft(FractalSetButtonNavigationPanel panel, double dStepSize) {
    if (panel.equals(m_pJuliaButtonNavigationPanel)) {
      m_pEffectPanel.moveLeftJulia(dStepSize);
    }
    else {
      m_pEffectPanel.moveLeftMandelbrot(dStepSize);
    }
  }
  public void moveRight(FractalSetButtonNavigationPanel panel, double dStepSize) {
    if (panel.equals(m_pJuliaButtonNavigationPanel)) {
      m_pEffectPanel.moveRightJulia(dStepSize);
    }
    else {
      m_pEffectPanel.moveRightMandelbrot(dStepSize);
    }
  }

  //////////////////////////////////////////
  // JULIA SET PARAMETER C PANEL LISTENER //
  //////////////////////////////////////////
  public void setParameterC(JuliaSetParameterCPanel panel, double dReal, double dImaginary) {
    m_pEffectPanel.setCParameter(dReal, dImaginary);
  }

  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void handleColorGradientPanelSet(JPanel currentPropertyPanel) {
    if (isColorGradientPanel(currentPropertyPanel)) {
      updateColorGradientPanelWidth((ColorGradientPanel)currentPropertyPanel);
    }
  }

  private boolean isColorGradientPanel(JPanel currentPropertyPanel) {
    if (currentPropertyPanel == null) return false;
    if (currentPropertyPanel.equals(m_pJuliaColorGradient)) return true;
    if (currentPropertyPanel.equals(m_pMBColorGradient)) return true;
    return false;
  }

  private void updateColorGradientPanelWidth(ColorGradientPanel colorGradientPanel) {
    colorGradientPanel.setPreferredSize(new Dimension(getColorGradientPanelWidth(), 50));
    colorGradientPanel.repaint();
    colorGradientPanel.revalidate();
  }

  private int getColorGradientPanelWidth() {
    return getWidth() - 50 - getPropertiesSelectionWidth();
  }











}
