package gui.transformation;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import transformation.ImageResizeParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.image.ImageLoadPanel;
import gui.image.IImageLoadPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static transformation.ImageResizeParameters.*;


/**
 *  A properties panel for the image resizing effect.
 **/
public class ImageResizePropertiesPanel extends AbstractPropertiesPanel
  implements ISliderPanelListener, IImageLoadPanelListener {

  /*** member variables ***/
  private ImageResizeParameters m_pImageResizeParameters;

  /*** gui member variables ***/
  private ImageLoadPanel m_pImageLoadPanel;
  private SliderPanel m_pRatioResizeSliderPanel;
  private JPanel m_pRatioResizePanel;
  private JLabel m_pRatioResizeLabel, m_pRatioResizeInformation;


  private JComboBox<String> m_pRatioResizeTypes;

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code ImageResizePropertiesPanel} object with the default parameter settings for the image resize effect.
   *
   **/
  public ImageResizePropertiesPanel() {
    //1. init members
    m_pImageResizeParameters = new ImageResizeParameters();

    //3. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addPanelListener(this);

    //4. ratio resize panel
    m_pRatioResizePanel = createRatioResizePanel();

    //5. init layout
    initLayout();

  }

  ///////////////////////////
  // PRIVATE GUI METHODS   //
  ///////////////////////////
  private JPanel createRatioResizePanel() {
    //1. create the panel and the layout
    JPanel pPanel = new JPanel();
    pPanel.setLayout(new FlowLayout());

    //2. add a label
    m_pRatioResizeLabel = new JLabel("Adapt:");
    pPanel.add(m_pRatioResizeLabel);

    //3. add the ratio type combo box
    Vector<String> pRatioTypes = new Vector<String>();
    pRatioTypes.add("Width");
    pRatioTypes.add("Height");
    pRatioTypes.add("Longer side");
    pRatioTypes.add("Shorter side");

    m_pRatioResizeTypes = new JComboBox<String>(pRatioTypes);
    m_pRatioResizeTypes.addActionListener(this);
    pPanel.add(m_pRatioResizeTypes);

    //4. the resizing value
    m_pRatioResizeSliderPanel = new IntegerSliderPanel(0, 50000, 1024);
    m_pRatioResizeSliderPanel.addPanelListener(this);
    pPanel.add(m_pRatioResizeSliderPanel);

    //5. Ratio resize information:
    m_pRatioResizeInformation = new JLabel();
    pPanel.add(m_pRatioResizeInformation);


    return pPanel;
  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code ImageResizeParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code ImageResizeParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pImageResizeParameters;
  }

  /**
   *  Returns the name of the image file that should be resized.
   *
   *  @return The name of the image file that should be resized.
   **/
  public String getFileName() {
    return m_pImageLoadPanel.getFileName();
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pImageLoadPanel);
    pPanels.add(m_pRatioResizePanel);
    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add("ImageResize.Image");
    pKeys.add("ImageResize.MaximumSide");
    return pKeys;
  }


  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////

  /**
   *  Sets the effect parameters that should be represented by this {@code ImageResizePropertiesPanel} object.
   *  The set operation is only successful if a {@code ImageResizeParameters} object has been specified.
   *
   *  @param pParameters The {@code ImageResizeParameters} object representing the effect parameters
   *                     that should be represented by this {@code ImageResizePropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof ImageResizeParameters) {
      m_pImageResizeParameters = (ImageResizeParameters)pParameters;

      updateControls();
    }
  }



  ////////////////////////////////////////
  // SLIDER PANEL LISTENER              //
  ////////////////////////////////////////
  /**
   *  If the resize slider has been modified, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pImageResizeParameters == null) m_pImageResizeParameters = new ImageResizeParameters();

    //2. set the new parameters
    if (pSliderPanel.equals(m_pRatioResizeSliderPanel)) {
      //setRatioLongerSideSettings();
      handleRatioResizeAction();
    }



    //2. fire the apply changes event
    fireApplyChanges(m_pImageResizeParameters);
  }






  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image that should be resized is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image on which the transition is applied.
   *  @param pImage The image on which the color transition should be painted.
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pImageResizeParameters == null) m_pImageResizeParameters = new ImageResizeParameters();

    //1. set the new image
    m_pImageResizeParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);
    handleRatioResizeAction();

    //2. fire the apply changes event
    fireApplyChanges(m_pImageResizeParameters);
  }

  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
   /**
   *   Applies the new setting of a ratio resize type respectively detects
   *   which parameter should be shown now in the properties panel.
   *
   *   @param pEvent The action event triggering this method.
   **/
  protected void actionPerformedForChildren(ActionEvent pEvent) {

    //1. if it is the ratio resize types panel -> handle ratio resize
    if (pEvent.getSource().equals(m_pRatioResizeTypes)) {
      handleRatioResizeAction();
      fireApplyChanges(m_pImageResizeParameters);
      return;
    }


  }

  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  protected void applyNewSettingsForChildren(Settings pSettings) {




     //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }

  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pImageResizeParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    //m_pBrickColorDisplay.setColor(m_pImageResizeParameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }

  private void handleRatioResizeAction() {
    //1. get the information from the loaded image
    BufferedImage pImage = (BufferedImage)m_pImageResizeParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE);
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();


    //2. get the value from the resize slider panel
    int iValue = m_pRatioResizeSliderPanel.getIntValue();


    //2. make the new size settings
    int iRatioResizeType = m_pRatioResizeTypes.getSelectedIndex();
    Dimension pNewSize = null;
    if (iRatioResizeType == 0) {
      pNewSize = getRatioWidthSettings(iWidth, iHeight, iValue);
    }
    else if (iRatioResizeType == 1) {
      pNewSize = getRatioHeightSettings(iWidth, iHeight, iValue);
    }
    else if (iRatioResizeType == 2) {
      pNewSize = getRatioLongerSideSettings(iWidth, iHeight, iValue);
    }
    else {
      pNewSize = getRatioShorterSideSettings(iWidth, iHeight, iValue);
    }

    //3. set the resize information
    m_pRatioResizeInformation.setText("(" + pNewSize.width + " x " + pNewSize.height + ")");
    m_pImageResizeParameters.setIntegerParameterValue(PARAM_INT_NEW_WIDTH, pNewSize.width);
    m_pImageResizeParameters.setIntegerParameterValue(PARAM_INT_NEW_HEIGHT, pNewSize.height);

  }

  private Dimension getRatioWidthSettings(int iWidth, int iHeight, int iValue) {

    //1. calculate the new size and set it in the properties
    int iNewWidth = iValue;
    double dNewHeight = (double)iValue/(double)iWidth*(double)iHeight;
    int iNewHeight = (int)dNewHeight;


    //2. return the new size for the information label
    return new Dimension(iNewWidth, iNewHeight);
  }


  private Dimension getRatioHeightSettings(int iWidth, int iHeight, int iValue) {

    //1. calculate the new size and set it in the properties
    int iNewHeight = iValue;
    double dNewWidth = (double)iValue/(double)iHeight*(double)iWidth;
    int iNewWidth = (int)dNewWidth;


    //2. return the new size for the information label
    return new Dimension(iNewWidth, iNewHeight);
  }

  private Dimension getRatioLongerSideSettings(int iWidth, int iHeight, int iValue) {


    //1. calculate the new size and set it in the properties
    int iNewWidth = 0;
    int iNewHeight = 0;
    //1.1. size according to the width (= longer side)
    if (iWidth > iHeight) {
      double dNewHeight = (double)iValue/(double)iWidth*(double)iHeight;
      iNewWidth = iValue;
      iNewHeight = (int)dNewHeight;
    }
    //1.2. size according to the height (= longer side)
    else {
      double dNewWidth = (double)iValue/(double)iHeight*(double)iWidth;
      iNewWidth = (int)dNewWidth;
      iNewHeight = iValue;
    }

    //2. return the new size for the information label
    return new Dimension(iNewWidth, iNewHeight);
  }



  private Dimension getRatioShorterSideSettings(int iWidth, int iHeight, int iValue) {


    //1. calculate the new size and set it in the properties
    int iNewWidth = 0;
    int iNewHeight = 0;
    //1.1. size according to the width (= shorter side)
    if (iWidth < iHeight) {
      double dNewHeight = (double)iValue/(double)iWidth*(double)iHeight;
      iNewWidth = iValue;
      iNewHeight = (int)dNewHeight;
    }
    //1.2. size according to the height (= shorter side)
    else {
      double dNewWidth = (double)iValue/(double)iHeight*(double)iWidth;
      iNewWidth = (int)dNewWidth;
      iNewHeight = iValue;
    }

    //2. return the new size for the information label
    return new Dimension(iNewWidth, iNewHeight);
  }





}
