package gui.transparency;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import standard.LinearTransitionProperties;
import transparency.ImageTransitionParameters;
import gui.transparency.LinearTransitionPanel;
import gui.transparency.ILinearTransitionPanelListener;
import gui.AbstractPropertiesPanel;
import gui.image.MultipleImageLoadPanel;
import gui.image.IMultipleImageLoadPanelListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static transparency.ImageTransitionParameters.*;
import static applications.ImageTransitionLanguageProperties.*;


/**
 *  A properties panel for the image transition effect.
 **/
public class ImageTransitionPropertiesPanel extends AbstractPropertiesPanel
  implements IMultipleImageLoadPanelListener, ILinearTransitionPanelListener, ActionListener {

  /*** member variables ***/
  private ImageTransitionParameters m_pImageTransitionParameters;

  /*** gui member variables ***/
  private MultipleImageLoadPanel m_pImageLoadPanel;
  private LinearTransitionPanel m_pTransitionCoordinatesPanel;


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code ImageTransitionPropertiesPanel} object with the default parameters
   *  of the {@code ImageTransition} effect.
   **/
  public ImageTransitionPropertiesPanel() {
    //1. init members
    m_pImageTransitionParameters = new ImageTransitionParameters();


    //2. load image panel
    m_pImageLoadPanel = new MultipleImageLoadPanel(2);
    m_pImageLoadPanel.addPanelListener(this);


    //3. transition coordinates panel
    LinearTransitionProperties pProperties = m_pImageTransitionParameters.getLinearTransitionProperties();
    m_pTransitionCoordinatesPanel = new LinearTransitionPanel(pProperties);
    m_pTransitionCoordinatesPanel.addPanelListener(this);

    initLayout();
  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code ImageTransitionParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code ImageTransitionParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pImageTransitionParameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pImageLoadPanel);
    pPanels.add(m_pTransitionCoordinatesPanel);


    return pPanels;

  }


  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_IMAGES);
    pKeys.add(KEY_TRANSITION_COORDINATES);

    return pKeys;
  }


  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code ImageTransitionPropertiesPanel} object.
   *  The set operation is only successful if a {@code ImageTransitionParameters} object has been specified.
   *
   *  @param pParameters The {@code ImageTransitionParameters} object representing the effect parameters
   *                     that should be represented by this {@code ImageTransitionPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof ImageTransitionParameters) {
      m_pImageTransitionParameters = (ImageTransitionParameters)pParameters;

      updateControls();
    }
  }






  /////////////////////////////////////////
  // MULTIPLE IMAGE LOAD PANEL LISTENER  //
  /////////////////////////////////////////
  /**
   *  If the image where the color transition should be painted is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code MultipleImageLoadPanel} object that steers the image on which the transition is applied.
   *  @param pImage The image on which the color transition should be painted.
   **/
  public void applyChanges(MultipleImageLoadPanel pPanel, int iImageId, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pImageTransitionParameters == null) m_pImageTransitionParameters = new ImageTransitionParameters();

    //1. set the new colors
    if (iImageId == 0) {
      m_pImageTransitionParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE1, pImage);
    }
    else {
      m_pImageTransitionParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE2, pImage);
    }

    //2. fire the apply changes event
    fireApplyChanges(m_pImageTransitionParameters);
  }

  /////////////////////////////////////////
  // LINEAR TRANSITION PANEL LISTENER    //
  /////////////////////////////////////////

  /**
   *  This method is called when the settings of the {@code LinearTransitionPanel} object have been changed.
   *
   *  @param pPanel The {@code LinearTransitionPanel} to which the listener is registered and that triggered this method call.
   *  @param pProperties The properties of the linear transition that are hold by the {@code LinearTransitionPanel}.
   */
  public void applyChanges(LinearTransitionPanel pPanel, LinearTransitionProperties pProperties) {
    m_pImageTransitionParameters.setLinearTransitionProperties(pProperties);
    fireApplyChanges(m_pImageTransitionParameters);
  }



  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }

    if (m_pTransitionCoordinatesPanel != null) {
      m_pTransitionCoordinatesPanel.applyNewSettings(pSettings);
    }


  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pImageTransitionParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    //m_pBrickColorDisplay.setColor(m_pImageTransitionParameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }






}
