package gui.transparency;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import java.awt.FlowLayout;
import java.awt.Dimension;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

//library imports
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;
import standard.LinearTransitionProperties;
import gui.JPanelWithListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;


import static applications.GuiLanguageProperties.*;

/**
 *  A panel with a label for an effect property and {@code ComboBox} managing the value of this effect property.
 **/
public class LinearTransitionPanel extends JPanelWithListener<ILinearTransitionPanelListener> 
  implements ChangeListener, ISliderPanelListener {
  
  /** private gui member variables **/
  private JLabel m_pStartLabel, m_pFiftyPercLabel, m_pEndLabel;
  private JSpinner m_pStartX, m_pStartY, m_pEndX, m_pEndY;
  private SliderPanel m_pFiftyPercPanel;
  
  private LinearTransitionProperties m_pProperties;
  
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates the {@code ComboBox} with a label text, the possible values for the {@code ComboBox} and the selected index to 
   *  indicate the current value of the {@code ComboBox}.
   *
   *  @param pProperties TODO 
   **/
  public LinearTransitionPanel(LinearTransitionProperties pProperties) {
    //layout
    setLayout(new FlowLayout());
    
    //start position
    m_pStartLabel = new JLabel();
    add(m_pStartLabel);
    
    
    
    Integer pMin = new Integer(-50000);
    Integer pMax = new Integer(50000);
    Integer pStep = new Integer(1);
    
    m_pStartX = new JSpinner(new SpinnerNumberModel(new Integer(0), pMin, pMax, pStep));
    m_pStartX.addChangeListener(this);
    add(m_pStartX);
    
    m_pStartY = new JSpinner(new SpinnerNumberModel(new Integer(200), pMin, pMax, pStep));
    m_pStartY.addChangeListener(this);
    add(m_pStartY);
    
    //half transparency slider panel
    m_pFiftyPercLabel = new JLabel();
    add(m_pFiftyPercLabel);
    
    m_pFiftyPercPanel = new IntegerSliderPanel(0, 100, 50);
    m_pFiftyPercPanel.addPanelListener(this);
    add(m_pFiftyPercPanel);
    
    //end position
    m_pEndLabel = new JLabel();
    add(m_pEndLabel);
    
    
    m_pEndX = new JSpinner(new SpinnerNumberModel(new Integer(400), pMin, pMax, pStep));
    m_pEndX.addChangeListener(this);
    add(m_pEndX);
    
    m_pEndY = new JSpinner(new SpinnerNumberModel(new Integer(200), pMin, pMax, pStep));
    m_pEndY.addChangeListener(this);
    add(m_pEndY);
    
    
    //TODO init the whole layout
    //m_pLabel = new JLabel(pText);
    //add(m_pLabel);
    
    setLinearTransitionProperties(pProperties);
    
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    setLabels(pLanguageProperties);
    
    
   
    
    
  
  }
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *  Returns the selected index of the {@code ComboBox}.
   *
   *  @return The selected index of the {@code ComboBox}.
   **/
  public LinearTransitionProperties getLinearTransitionProperties() {
    return m_pProperties;
  }
  

  
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the text of the label. 
   *
   *  @param pText The new text of the label as String.
   **/
  public void setLinearTransitionProperties(LinearTransitionProperties pProperties) {
    if (pProperties == null) return;
    m_pProperties = pProperties;
    
    //TODO adapt the gui elements
  }
  
 
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    
    
    //3. fetch the new parameter value
    Object pSource = e.getSource();
    
    
    if (pSource.equals(m_pStartX)) {
      m_pProperties.setStartX(((Integer)m_pStartX.getValue()).intValue());
    }
    else if (pSource.equals(m_pStartY)) {
      m_pProperties.setStartY(((Integer)m_pStartY.getValue()).intValue());
    }
    else if (pSource.equals(m_pEndX)) {
      m_pProperties.setEndX(((Integer)m_pEndX.getValue()).intValue());
    }
    else if (pSource.equals(m_pEndY)) {
      m_pProperties.setEndY(((Integer)m_pEndY.getValue()).intValue());
    }
    
    
  
    
    
    //4. the apply changes event
    applyChanges();

  }
  
  
  ///////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER        //
  ///////////////////////////////////////
  /**
   *  If the number of spikes, the spike length, the line number, the translation or rotation values have been adapted 
   *  in the corresponding slider panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pProperties == null) return;
    
    //TODO: set up the slider panel as double slider panel
    m_pProperties.setFiftyPercentTranparencyPos(dValue/100);
    
    
    
    //2. fire the apply changes event
    applyChanges();
  }
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setLabels(pLanguageProperties);
    
    
    
    
    
    
    
  }
  
 
  
  ///////////////////////////////
  // LISTENER METHODS          //
  ///////////////////////////////
  protected void applyChangesToListener(ILinearTransitionPanelListener pListener) {
    pListener.applyChanges(this, m_pProperties);
  }
  
  
  
  /////////////////////////////////
  // PRIVATE HELPERS             //
  /////////////////////////////////
  private void setLabels(LanguageProperties pLanguageProperties) {
    m_pStartLabel.setText(pLanguageProperties.getProperty(KEY_LINEAR_TRANSITION_PANEL_START));
    m_pFiftyPercLabel.setText(pLanguageProperties.getProperty(KEY_LINEAR_TRANSITION_PANEL_FIFTYPERC));
    m_pEndLabel.setText(pLanguageProperties.getProperty(KEY_LINEAR_TRANSITION_PANEL_END));
  }
  
  
  
  
}