package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.io.File;

//library imports
import standard.ImageSequence;
import standard.IVideoEffectListener;


/**
 *  Saves the results of video renderings.
 **/
public class RenderingOutput implements IVideoEffectListener {

  /*** member variables ***/
  private int m_iFirstFrame, m_iFileType;
  private String m_pFileBaseName, m_pDirectory;
  private boolean m_bLeadingZeros, m_bCreateDirectories, m_bOverwriteImageFiles;
  
  /*** constants for the default settings ***/
  private static final int FIRSTFRAME = 1;
  private static final int FILETYPE = ImageSequence.FILE_TYPE_PNG;
  private static final String FILEBASENAME = "rendered";
  private static final String DIRECTORY = "";
  private static final boolean LEADING_ZEROS = false;
  private static final boolean CREATE_DIRECTORIES = false;
  private static final boolean OVERWRITE_IMAGE_FILES = false;


  ////////////////////////////
  // CONSTRUCTOR            //
  ////////////////////////////
  /**
   *  Creates a rendering output with the default settings for saving the image files. (Ie. the first frame, 
   *  the file type, the file base name, the directory, no leading zeros in the image count, no directories are created, 
   *  existing images are not overwritten.
   **/
  public RenderingOutput() {
    m_iFirstFrame = FIRSTFRAME;
    m_iFileType = FILETYPE;
    
    m_pFileBaseName = FILEBASENAME;
    m_pDirectory = DIRECTORY;
    
    m_bLeadingZeros = LEADING_ZEROS;
    m_bCreateDirectories = CREATE_DIRECTORIES;
    m_bOverwriteImageFiles = OVERWRITE_IMAGE_FILES;
    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the id of the first frame. This id must not be smaller than 0.
   *
   *  @param iFirstFrame The id of the first frame as integer.
   **/
  public void setFirstFrame(int iFirstFrame) {
    if (iFirstFrame < 0) return;
    m_iFirstFrame = iFirstFrame;
  }
  
  /**
   *  Sets the file type. Valid file types are defined as the file type constants in the {@code ImageSequence} class.
   *  If no valid file type is defined, the PNG file format is chosen.
   *
   *  @param iFileType The file type as integer.
   **/
  public void setFileType(int iFileType) {
    if (iFileType == ImageSequence.FILE_TYPE_JPG) {
      m_iFileType = ImageSequence.FILE_TYPE_JPG;
    }
    else if (iFileType == ImageSequence.FILE_TYPE_BMP) {
      m_iFileType = ImageSequence.FILE_TYPE_BMP;
    }
    else {
      m_iFileType = ImageSequence.FILE_TYPE_PNG;
    }
  
  }
  
  /**
   *  Sets the file base name of the output images that should be saved. {@code null} values are neglected.
   *
   *  @param pFileBaseName The file base name as string.
   **/
  public void setFileBaseName(String pFileBaseName) {
    if (pFileBaseName == null) return;
    m_pFileBaseName = pFileBaseName;
  }
  
  /**
   *  Sets the directory where the output images should be saved. {@code null} values are neglected.
   *
   *  @param pDirectory The directory as string.
   **/
  public void setDirectory(String pDirectory) {
    if (pDirectory == null) return;
    m_pDirectory = pDirectory;
  }
  
  /**
   *  Sets if leading zeros should be used in the image count.
   *
   *  @param bLeadingZeros The flag if leading zeros in the image count should be used as boolean.
   **/
  public void useLeadingZeros(boolean bLeadingZeros) {
    m_bLeadingZeros = bLeadingZeros;
  }
  
  /**
   *  Sets if a directory should be created, if the specified directory does not exist.
   *
   *  @param bCreateDirectories The flag if a directory should be created, if the specified directory does not exist as boolean.
   **/
  public void createDirectories(boolean bCreateDirectories) {
    m_bCreateDirectories = bCreateDirectories;
  }
  
  /**
   *  Sets if already existing image files should be overwritten.
   *
   *  @param bOverwriteImageFiles The flag if already existing image files should be overwritten as boolean.
   **/
  public void overwriteImageFiles(boolean bOverwriteImageFiles) {
    m_bOverwriteImageFiles = bOverwriteImageFiles;
  }
  
  
  
  
  
  
  
  

  ////////////////////////////////////
  // IVIDEOEFFECTLISTENER INTERFACE //
  ////////////////////////////////////
  
  /**
   *  Saves the specified image according to the settings made in this object.
   *
   *  @param pCurrentFrame The video frame that should be saved as {@code BufferedImage} object.
   *  @param iFrameId The id of the current frame as integer.
   *  @param iFrameNumber The total number of frames as integer.
   **/
  public void setCurrentVideoFrame(BufferedImage pCurrentFrame, int iFrameId, int iFrameNumber) {
  
    //1. check if the destination folder exists
    // TO DO: This check can be moved into a processingStarted method. Then it is not repeated all the time.
    if (m_pDirectory == null) m_pDirectory = "";
    m_pDirectory = m_pDirectory.trim();
    
    if (!m_pDirectory.equals("")) {
      
      File pFolder = new File(m_pDirectory);
      if (!pFolder.exists()) {
        if (m_bCreateDirectories) {
          boolean bCreated = false;
          String pExceptionMessage = "";
          try {
            bCreated = pFolder.mkdirs();
          }
          catch (Exception pException) {
            bCreated = false;
            pExceptionMessage = pException.getMessage();
          }
          
          if (!bCreated) {
            System.out.println("The directory '" + m_pDirectory + "' could not be created! " + 
            pExceptionMessage + " No file is saved!");
            return; //TO DO: find a good Exception Handling
          }
          
        }
        else {
          System.out.println("The directory '" + m_pDirectory + "' does not exist! No file is saved!");
          return; //TO DO: find a good Exception Handling
        }
      }
    }
    
    //2. check if the current file exists
    String pFullPath = "";
    if (!m_pDirectory.equals("")) pFullPath += m_pDirectory;
    if (!pFullPath.endsWith(File.separator)) pFullPath += File.separator;
    pFullPath += m_pFileBaseName;
    String pFileNameForSaving = pFullPath; // file name without the number part and the file type
    
    int iCurrentFrameId = m_iFirstFrame+iFrameId-1;
    String pCurrentFrameId = iCurrentFrameId+"";
    
    if (m_bLeadingZeros) {
      int iMaxFrameIdPositions = ((iFrameNumber+m_iFirstFrame-1)+"").length();
      
      while (pCurrentFrameId.length() < iMaxFrameIdPositions) {
        pCurrentFrameId = "0" + pCurrentFrameId;
        pFileNameForSaving += "0";// file name without the number part but with leading zeros and the file type
      }
    }
    pFullPath += pCurrentFrameId;
    
    String pFileType = ".png";
    if (m_iFileType == ImageSequence.FILE_TYPE_JPG) {
      pFileType = ".jpg";
    }
    else if (m_iFileType == ImageSequence.FILE_TYPE_BMP) {
      pFileType = ".bmp";
    }
    pFullPath += pFileType;
    
    File pImageFile = new File(pFullPath);
    if (pImageFile.exists()) {
      if (!m_bOverwriteImageFiles) {
        System.out.println("The file '" + pImageFile.getAbsolutePath() + "' does already exist! No file is saved!");
        return;
      }
    }
  
  
  
    //3. write the image
    ImageSequence.saveImage(pCurrentFrame, iCurrentFrameId, pFileNameForSaving, m_iFileType);
  }

  /**
   *  Not used at the moment.
   **/
  public void processingStarted() {}
  
  
  /**
   *   Not used.
   *
   *   @param dProgress    The percentage of the work that has already been finished.
   *   @param pCurrentWork A <code>String</code> explaning what the thread currently processes.
   *                       This <code>String</code> can also be <code>null</code>.
   **/
  public void setProgressStatus(double dProgress, String pCurrentWork) {}

  /**
   *   Not used.
   **/
  public void endedProperly() {}

  /**
   *   Not used.
   *
   *   @param pReason The reason, why the thread has been interrupted as <code>String</code>.
   **/
  public void interrupted(String pReason) {}


  

}