package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.image.BufferedImage;

/**
 *  Abstract class defining the interface for image effects and 
 *  implementing helping methods that are valid for all image effects.
 **/
public abstract class ImageEffect implements Runnable {

  /*** Threading member variables ***/
  private Thread m_pThread;
  private boolean m_bRun = true;
  private boolean m_bFinished = false;
  private IImageEffectListener m_pListener;

  //////////////////////////////////
  // PROCESS METHODS              //
  //////////////////////////////////
  /**
   *  Processes the image effect and returns the result image. 
   *
   *  @return The result image of the image effect as {@code BufferedImage} object.
   *
   *  @throws Exception if there are errors when processing the image effect.
   **/
  public abstract BufferedImage process() throws Exception;
  
  
  /**
   *  Processes the image effect in a thread. A listener has to be passed on that is waiting on the result of the image effect.
   *
   *  @param pListener A listener object that waits for the result of the image effect processing.
   **/
  public void processAsThread(IImageEffectListener pListener) {
    //1. make a simple check
    if (pListener == null) return;
    m_pListener = pListener;
  
    //2. when a thread is running, end it
    if (m_pThread != null) {
      m_bRun = false;
      while (!m_bFinished) {
        try {
          wait(20);
        }
        catch (Exception pException) {}
      }
    }

    
    //3. start a new thread
    m_pThread = new Thread(this);
    m_pThread.start();
  }
  
  
  //////////////////////////////////
  // SET METHODS                  //
  //////////////////////////////////
  /**
   *  Sets the image effect parameters that configure the effect.
   *
   *  @param pParameters The parameters for this image effect.
   **/
  public abstract void setEffectParameters(EffectParameters pParameters);
  
  //////////////////////////////////
  // GET METHODS                  //
  //////////////////////////////////
  
  /**
   *  Returns the image effect parameters that configure the effect.
   *
   *  @return The parameters for this image effect.
   **/
  public abstract EffectParameters getEffectParameters();
  
  
  /**
   *  Returns a boolean indicating if this image effect is still processing.
   *
   *  @return A boolean indicating if this image effect is still processing.
   **/
  public boolean isRunning() {
    return m_bRun;
  }
  
  //////////////////////////////////
  // RUNNABLE INTERFACE           //
  //////////////////////////////////
  /**
   *  Implements the Thread execution of this image effect.
   *
   **/
  public void run() {
    
    //1. set the thread members
    m_bFinished = false;
    m_bRun = true;
    
    //2. do the processing
    BufferedImage pImage = null;
    try {
      pImage = process();
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }
    m_bFinished = true;
    
    //3. check if the processing was completed and alert all listeners
    if (pImage != null) {
      m_pListener.setEffectImage(pImage);
    }
      
  }
  
  

}