package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.io.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import javax.imageio.ImageIO;

import java.util.Vector;



/** The class implements the access to the single frames of an image sequence.
 *  Also static routines for saving images or an image sequence are implemented.
 *
 */
public class ImageSequence implements IVideo {



  //filetypes
  /** Represents the file type ".bmp" */
  public static final int FILE_TYPE_BMP = 0;
  /** Represents the file type ".jpg" */
  public static final int FILE_TYPE_JPG = 1;
  /** Represents the file type ".png" */
  public static final int FILE_TYPE_PNG = 2;


  /** member variables for handling an image sequence with the same file base name **/
  private String m_pBaseFileName;
  private String m_pFileType;

  private int m_iStartFrame;
  private int m_iEndFrame;
  private int m_iNumberPositions;


  /** member variables for handling an image sequence with arbitrary image names **/
  private Vector<String> m_pFileNames;


  /** other member variables **/
  private int m_iFrameNumber;


  /**
   *  The constructor initializes the member variables specifying the filename, start and end frame,
   *  filetype, framenumber and number of positions of the running index in the image sequence. This constructor
   *  allows the processing of image sequences with filenames like:
   *  <ul><li>file1.bmp, file2.bmp, ..., file10.bmp, file11.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *      <li>file001.bmp, file002.bmp, ..., file010.bmp, file011.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pBaseFileName String indicating the filename before the index part of the image files.
   *  @param iStartFrame   Integer indicating the index of the first frame of the image sequence.
   *  @param iEndFrame     Integer indicating the index of the last frame of the image sequence.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   */
  public ImageSequence(String pBaseFileName, int iStartFrame, int iEndFrame, int iFileType) {

    //set member variables
    m_pBaseFileName = pBaseFileName;
    m_iStartFrame = iStartFrame;
    m_iEndFrame = iEndFrame;
    m_iFrameNumber = iEndFrame - iStartFrame + 1;

    m_pFileNames = null;


    //decide which filetype should be used
    if (iFileType == FILE_TYPE_BMP) {
      m_pFileType = ".bmp";
    }
    else if (iFileType == FILE_TYPE_JPG) {
      m_pFileType = ".jpg";
    }
    else {
      m_pFileType = ".png";
    }


    //check how many number positions are in the image sequence
    File pFirstFile = new File(m_pBaseFileName + m_iStartFrame + m_pFileType);


    if (pFirstFile.exists()) {
      m_iNumberPositions = -1;
    }
    else {

      String pZeroes = "0";

      //handle leading zeros
      while (pZeroes.length() < 10) {

        pFirstFile = new File(m_pBaseFileName + pZeroes + m_iStartFrame + m_pFileType);

        if (pFirstFile.exists()) break;

        pZeroes += "0";

      }

      //the files could not be found
      if (pZeroes.length() >= 10) {
        m_iStartFrame = 0;
        m_iEndFrame = -1;
        m_iFrameNumber = 0;

        /*Messages.printErrorMessage("ImageSequence",
        "The specified image files could not be found. The following absolute path was calculated: " +
        pFirstFile.getAbsolutePath());
        */

        return;
      }


      m_iNumberPositions = pZeroes.length() + (new String(m_iStartFrame+"")).length();

    }


  }


  /**
   *  The constructor allows the creation of an image sequence based on images with arbitrary filenames.
   *  To achieve this the filenames are given as a <code>Vector</code> of <code>String</code> objects. The order
   *  of these objects is not changed. Thus the position of an image in the sequence is determined by the position
   *  of its filename in the <code>Vector</code> object.
   *
   *  @param pFileNames The <code>Vector</code> holding the filenames of the images as <code>String</code> objects.
   */
  public ImageSequence(Vector<String> pFileNames) {

    //1. set unnecessary member variables
    m_pBaseFileName = null;
    m_pFileType = null;

    m_iStartFrame = 0;
    m_iEndFrame = 0;

    //2. make checks
    if (pFileNames == null) {
      m_pFileNames = null;
      m_iFrameNumber = 0;

      return;
    }

    //3. set important member variables
    m_pFileNames = pFileNames;
    m_iFrameNumber = pFileNames.size();



  }



  /**
   *  The constructor allows the creation of an image sequence based on images with arbitrary filenames.
   *  To achieve this the filenames are given as an array of <code>File</code> objects. The order
   *  of these objects is not changed. Thus the position of an image in the sequence is determined by the position
   *  of its filename in the array.
   *
   *  @param pFiles The array of <code>File</code> objects representing the images of the image sequence.
   */
  public ImageSequence(File[] pFiles) {

    //1. set unnecessary member variables
    m_pBaseFileName = null;
    m_pFileType = null;

    m_iStartFrame = 0;
    m_iEndFrame = 0;

    //2. make checks
    if ((pFiles == null) || (pFiles.length == 0)) {
      m_pFileNames = null;
      m_iFrameNumber = 0;

      return;
    }

    //3. set important member variables
    m_pFileNames = new Vector<String>();
    for (int i = 0; i < pFiles.length; i++) {
      m_pFileNames.add(pFiles[i].getAbsolutePath());
    }
    m_iFrameNumber = pFiles.length;



  }



  /**
   *  The constructor allows the creation of an image sequence based on the scheme of the first filename.
   *  The properties filebasename, startframe and filetype are extracted from the filename and the directory is searched
   *  for files that apply to these properties. Thus the number of frames is automatically detected. Gaps in the numeration
   *  of the files are not compensated.<br><br>
   *  This constructor
   *  allows the processing of image sequences with filenames like:
   *  <ul><li>file1.bmp, file2.bmp, ..., file10.bmp, file11.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *      <li>file001.bmp, file002.bmp, ..., file010.bmp, file011.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  <br><br>
   *  Also the use of a single image file, can be realized with this constructor, by passing a file containing no number part
   *  in its filename.
   *
   *  @param pFirstImageFile The <code>File</code> object representing the first frame of the image sequence.
   */
  public ImageSequence(File pFirstImageFile) {
    //1. set unnecessary member variables
    m_pBaseFileName = null;
    m_pFileType = null;

    m_iStartFrame = 0;
    m_iEndFrame = 0;

    //2. make checks
    if ((pFirstImageFile == null) || (!(pFirstImageFile.exists()))) {
      m_pFileNames = null;
      m_iFrameNumber = 0;

      return;
    }






    //3. analyze the filename
    String m_pAbsolutePath = pFirstImageFile.getAbsolutePath();


    //3.1. extract file type
    int iPosition = m_pAbsolutePath.length() - 1;
    while (!(Character.isDigit(m_pAbsolutePath.charAt(iPosition)))) {
      iPosition--;

      //file contains no number -> a single image
      if (iPosition < 0) {
        m_pFileNames = new Vector<String>();
        m_pFileNames.add(m_pAbsolutePath);
        m_iFrameNumber = 1;
        return;
      }
    }

    String pFileType = m_pAbsolutePath.substring(iPosition+1);




    //3.2. determine number and filebasename
    int iPosition2 = iPosition-1;
    while (Character.isDigit(m_pAbsolutePath.charAt(iPosition2))) {
      iPosition2--;
    }

    String pNumber = m_pAbsolutePath.substring(iPosition2+1, iPosition+1);
    String pFileBaseName = m_pAbsolutePath.substring(0, iPosition2+1);

    //3.3. validate the number and check if the number part has leading zeros
    int iStartNum = -1;
    try {
      iStartNum = Integer.parseInt(pNumber);
    }
    catch (Exception e) {
      m_pFileNames = new Vector<String>();
      m_pFileNames.add(m_pAbsolutePath);
      m_iFrameNumber = 1;
      return;
    }


    int iDigitNumber = pNumber.length();
    boolean bLeadingZeros = (iDigitNumber > 1);
    if (pNumber.charAt(0) != '0') bLeadingZeros = false;





    //4. add the files to the Vector of strings
    m_pFileNames = new Vector<String>();
    m_pFileNames.add(m_pAbsolutePath); //add first image



    //4.1. create file name
    int iCurrNum = iStartNum+1;
    String pCurrFileName = pFileBaseName + iCurrNum + pFileType;

    if (bLeadingZeros) {
      String pCurrentNumber = iCurrNum+"";
      while (pCurrentNumber.length() < iDigitNumber) {
        pCurrentNumber = "0"+pCurrentNumber;
      }

      pCurrFileName = pFileBaseName + pCurrentNumber + pFileType;

    }


    File pCurrentFile = new File(pCurrFileName);




    //4.2. iterate until end
    while (pCurrentFile.exists()) {

      m_pFileNames.add(pCurrFileName);

      iCurrNum++;
      pCurrFileName = pFileBaseName + iCurrNum + pFileType;

      if (bLeadingZeros) {
        String pCurrentNumber = iCurrNum+"";
        while (pCurrentNumber.length() < iDigitNumber) {
          pCurrentNumber = "0"+pCurrentNumber;
        }

        pCurrFileName = pFileBaseName + pCurrentNumber + pFileType;

      }


      pCurrentFile = new File(pCurrFileName);


    }



    m_iFrameNumber = m_pFileNames.size();



  }



  /**
   *  Returns the number of frames in the image sequence.
   *
   *  @return The number of frames in the image sequence as integer.
   */
  public int getFrameNumber() {
    return m_iFrameNumber;
  }



 /**
   *  Returns the frame of the image sequence with the given frameID.
   *
   *  @param iFrameID ID of the frame that should be returned as integer. Valid values are from 0 to frameNumber - 1.
   *
   *  @return The frame of the image sequence with the given frameID as BufferedImage object or null if an error occurred.
   */
  public BufferedImage getFrame(int iFrameID) {
    if (iFrameID < 0) return null;
    if (iFrameID >= m_iFrameNumber) return null;

    String pFilename = null;


    //file names are not explicitely given -> we have to calculate them
    if (m_pFileNames == null) {

      pFilename = m_pBaseFileName;
      String pNumber = (iFrameID+m_iStartFrame) + "";

      while (pNumber.length() < m_iNumberPositions) {
        pNumber = "0" + pNumber;
      }

      pFilename += pNumber + m_pFileType;
    }

    //file names are explicitely given
    else {
      pFilename = m_pFileNames.get(iFrameID);
    }



    //try to load the current frame
    try {
      return ImageIO.read(new File(pFilename));
    }
    catch (Exception e) {
      System.out.println("Image " + iFrameID + " ("+pFilename+") could not be read!");
      e.printStackTrace();
      return null;
    }

  }



 /**
   *  Saves a given image into a file with the given filebasename and the given index. The filetype
   *  can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file1.bmp, file2.bmp, ..., file10.bmp, file11.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage        BufferedImage object representing the image that should be saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *
   *  @return false, if an error occurred.
   */
  public static boolean saveImage(BufferedImage pImage, int iFrameID, String pFileBaseName, int iFileType) {

    if (iFrameID < 0) return false;


    String pFileTypeString;

    //1. decide which filetype should be used
    if (iFileType == FILE_TYPE_BMP) {
      pFileTypeString = "bmp";
    }
    else if (iFileType == FILE_TYPE_JPG) {
      pFileTypeString = "jpg";
    }
    else {
      pFileTypeString = "png";
    }


    //2. save the image
    try {
      File pImageFile = new File(pFileBaseName + iFrameID + "." + pFileTypeString);
      ImageIO.write(pImage, pFileTypeString, pImageFile);
    }
    catch (Exception e) {
      //TO DO: include the message of the exception into the error print out.
      System.out.println("Image " + iFrameID + " could not be saved!");
      e.printStackTrace();
      return false;
    }

    return true;


  }


  /**
   *  Saves a given image into a file defined by a parameter of the type {@code File}.
   *
   *  @param pImage        BufferedImage object representing the image that should be saved.
   *  @param pFile         The {@code File} object that defines into which file the image should be saved.
   *
   *  @return false, if an error occurred.
   */
 
  public static boolean saveImage(BufferedImage pImage, File pFile) {
    //1. make some checks
    if (pImage == null) return false;
    if (pFile == null) return false;

    //2. check which file type is used
    String pFilename = pFile.getAbsolutePath();
    String pFileTypeString = pFilename.substring(pFilename.length()-3);
    pFileTypeString = pFileTypeString.toLowerCase();
    
    boolean bIsValidFileType = false;
    if (pFileTypeString.equals("png")) bIsValidFileType = true;
    if (pFileTypeString.equals("gif")) bIsValidFileType = true;
    if (pFileTypeString.equals("jpg")) bIsValidFileType = true;
    if (!bIsValidFileType) {
      pFilename += ".png";
      pFileTypeString = "png";
      pFile = new File(pFilename);
    }
    
    //3. save the image
    try {
      ImageIO.write(pImage, pFileTypeString, pFile);
    }
    catch (Exception e) {
      //TODO: include the message of the exception into the error print out.
      //System.out.println("Image " + iFrameID + " could not be saved!");
      e.printStackTrace();
      return false;
    }

    return true;

  }



  /**
   *  Saves a given image into a file with the given filebasename and the given index. The filetype
   *  can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file001.bmp, file002.bmp, ..., file010.bmp, file011.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage        BufferedImage object representing the image that should be saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *  @param iFrameNum     The number of frames that the image sequence has.
   *
   *  @return false, if an error occurred.
   */
  public static boolean saveImageWithLeadingZeros(  BufferedImage pImage,
                                                    int iFrameID,
                                                    String pFileBaseName,
                                                    int iFileType,
                                                    int iFrameNum)
  {
      if (iFrameID > iFrameNum) return false;
      if (iFrameID < 0) return false;

      //1. calculate the number of leading zeros and update the file base name
      String pFrameID = iFrameID+"";
      String pFrameNum = iFrameNum+"";

      int iFrameIDLength = pFrameID.length();
      int iFrameNumLength = pFrameNum.length();

      int iLeadingZeroNum = iFrameNumLength - iFrameIDLength;

      for (int i = 0; i < iLeadingZeroNum; i++) {
        pFileBaseName += "0";
      }

      //2. call the saveImage method
      return saveImage(pImage, iFrameID, pFileBaseName, iFileType);

  }



 /**
   *  Combines the two given images into one and saves the combined image into a file with the given filebasename and
   *  the given index. The filetype can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file1.bmp, file2.bmp, ..., file10.bmp, file11.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage1       BufferedImage object representing the first image that should be combined and saved.
   *  @param pImage2       BufferedImage object representing the second image that should be combined and saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *
   *  @return false, if an error occurred.
   */
  public static boolean combineAndSaveTwoImages(    BufferedImage pImage1,
                                                    BufferedImage pImage2,
                                                    int iFrameID,
                                                    String pFileBaseName,
                                                    int iFileType)
  {


    //1. check if any image is null
    if ((pImage1 == null) || (pImage2 == null)) {
      System.out.println("[Frame " + iFrameID + "]: The images could not be combined!");
      return false;
    }


    //2. combine the images
    int iWidth = pImage1.getWidth();
    int iHeight = pImage1.getHeight();

    BufferedImage pImage = new BufferedImage(2*iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();

    pGraphics.drawImage(pImage1, 0, 0, null);
    pGraphics.drawImage(pImage2, iWidth, 0, null);


    //3. save the combined image
    return saveImage(pImage, iFrameID, pFileBaseName, iFileType);

  }



  /**
   *  Combines the two given images into one and saves the combined image into a file with the given filebasename
   *  and the given index. The filetype
   *  can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file001.bmp, file002.bmp, ..., file010.bmp, file011.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage1       BufferedImage object representing the first image that should be combined and saved.
   *  @param pImage2       BufferedImage object representing the second image that should be combined and saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *  @param iFrameNum     The number of frames that the image sequence has.
   *
   *  @return false, if an error occurred.
   */
  public static boolean combineAndSaveTwoImagesWithLeadingZeros(  BufferedImage pImage1,
                                                                  BufferedImage pImage2,
                                                                  int iFrameID,
                                                                  String pFileBaseName,
                                                                  int iFileType,
                                                                  int iFrameNum)
  {
      if (iFrameID > iFrameNum) return false;
      if (iFrameID < 0) return false;

      //1. calculate the number of leading zeros and update the file base name
      String pFrameID = iFrameID+"";
      String pFrameNum = iFrameNum+"";

      int iFrameIDLength = pFrameID.length();
      int iFrameNumLength = pFrameNum.length();

      int iLeadingZeroNum = iFrameNumLength - iFrameIDLength;

      for (int i = 0; i < iLeadingZeroNum; i++) {
        pFileBaseName += "0";
      }

      //2. call the saveImage method
      return combineAndSaveTwoImages(pImage1, pImage2, iFrameID, pFileBaseName, iFileType);

  }




 /**
   *  Combines the four given images into one and saves the combined image into a file with the given filebasename and
   *  the given index. The filetype can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file1.bmp, file2.bmp, ..., file10.bmp, file11.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage1       BufferedImage object representing the first image that should be combined and saved.
   *  @param pImage2       BufferedImage object representing the second image that should be combined and saved.
   *  @param pImage3       BufferedImage object representing the third image that should be combined and saved.
   *  @param pImage4       BufferedImage object representing the fourth image that should be combined and saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *
   *  @return false, if an error occurred.
   */
  public static boolean combineAndSaveFourImages(   BufferedImage pImage1,
                                                    BufferedImage pImage2,
                                                    BufferedImage pImage3,
                                                    BufferedImage pImage4,
                                                    int iFrameID,
                                                    String pFileBaseName,
                                                    int iFileType)
  {


    //1. check if any image is null
    if ((pImage1 == null) || (pImage2 == null) || (pImage3 == null) || (pImage4 == null)) {
      System.out.println("[Frame " + iFrameID + "]: The images could not be combined!");
      return false;
    }


    //2. combine the images
    int iWidth = pImage1.getWidth();
    int iHeight = pImage1.getHeight();

    BufferedImage pImage = new BufferedImage(2*iWidth, 2*iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();

    pGraphics.drawImage(pImage1, 0, 0, null);
    pGraphics.drawImage(pImage2, iWidth, 0, null);
    pGraphics.drawImage(pImage3, 0, iHeight, null);
    pGraphics.drawImage(pImage4, iWidth, iHeight, null);


    //3. save the combined image
    return saveImage(pImage, iFrameID, pFileBaseName, iFileType);

  }




 /**
   *  Combines the four given images into one and saves the combined image into a file with the given filebasename
   *  and the given index. The filetype
   *  can also be chosen. This method allows the storage of image sequences with filenames like:
   *  <ul><li>file001.bmp, file002.bmp, ..., file010.bmp, file011.bmp, ..., file100.bmp, file101.bmp, ...</li>
   *  </ul>
   *
   *  @param pImage1       BufferedImage object representing the first image that should be combined and saved.
   *  @param pImage2       BufferedImage object representing the second image that should be combined and saved.
   *  @param pImage3       BufferedImage object representing the third image that should be combined and saved.
   *  @param pImage4       BufferedImage object representing the fourth image that should be combined and saved.
   *  @param iFrameID      Integer indicating the index of image file.
   *  @param pFileBaseName String indicating the filename before the index part of the image file.
   *  @param iFileType     The filetype of the images. Valid values are (FILE_TYPE_BMP, FILE_TYPE_JPG, FILE_TYPE_PNG)
   *  @param iFrameNum     The number of frames that the image sequence has.
   *
   *  @return false, if an error occurred.
   */
  public static boolean combineAndSaveFourImagesWithLeadingZeros( BufferedImage pImage1,
                                                                  BufferedImage pImage2,
                                                                  BufferedImage pImage3,
                                                                  BufferedImage pImage4,
                                                                  int iFrameID,
                                                                  String pFileBaseName,
                                                                  int iFileType,
                                                                  int iFrameNum)
  {
      if (iFrameID > iFrameNum) return false;
      if (iFrameID < 0) return false;

      //1. calculate the number of leading zeros and update the file base name
      String pFrameID = iFrameID+"";
      String pFrameNum = iFrameNum+"";

      int iFrameIDLength = pFrameID.length();
      int iFrameNumLength = pFrameNum.length();

      int iLeadingZeroNum = iFrameNumLength - iFrameIDLength;

      for (int i = 0; i < iLeadingZeroNum; i++) {
        pFileBaseName += "0";
      }

      //2. call the saveImage method
      return combineAndSaveFourImages(pImage1, pImage2, pImage3, pImage4, iFrameID, pFileBaseName, iFileType);

  }




}
