package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *  Provides interpolation methods for double values. 
 **/
public class InterpolatedDouble implements IInterpolated<InterpolatedDouble> {

  /*** member variables ***/
  private double m_dValue;

  /**
   *  Creates an {@code InterpolatedDouble} object with the specified double value.
   *
   *  @param dValue The double value that should be represented by this {@code InterpolatedDouble} object.
   **/
  public InterpolatedDouble(double dValue) {
    m_dValue = dValue;
  }
  
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the integer value that is nearest to the double value represented by this object.
   *
   *  @return The integer value that is nearest to the double value represented by this object.
  **/
  public int intValue() {
    return (int)Math.rint(m_dValue); //round to the nearest integer
  }
  
  /**
   *  Returns the double value that is represented by this object.
   *
   *  @return The double value that is represented by this object.
  **/
  public double doubleValue() {
    return m_dValue;
  }
  
  /**
   *  Returns a string representation of the double value that is represented by this object.
   * 
   *  @return a string representation of the double value that is represented by this object.
   **/
  public String toString() {
    return (m_dValue + "");
  }

  
  /////////////////////////////////
  // IINTERPOLATED INTERFACE     //
  /////////////////////////////////
  /**
   *  Returns an {@code InterpolatedDouble} object representing the double value which is an interpolation between the double value of this object, 
   *  and the double value that has been passed on as parameter. 
   *
   *  @param pOtherObject The second double value that is needed to calculate the interpolated value. 
   *  @param dLambda Steers the weights of the double values. A value of 1 means that the value of this object gets fully considered. 
   *                 A value of 0 means that the parameter value gets fully considered. (TO DO: Should not it be the other way round?)
   *
   *  @return The resulting double value of the interpolation.
   **/
  public InterpolatedDouble interpolateLinearly(InterpolatedDouble pOtherObject, double dLambda) {
    //1. make a simple check
    if (pOtherObject == null) return new InterpolatedDouble(m_dValue);
  
    //2. interpolate
    double dInterpolatedValue = (dLambda)*m_dValue + (1 -dLambda)*pOtherObject.doubleValue();
    return new InterpolatedDouble(dInterpolatedValue); 
  }

}