package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *  Provides interpolation methods for integer values. 
 **/
public class InterpolatedInteger implements IInterpolated<InterpolatedInteger> {

  /*** member variables ***/
  private int m_iValue;

  ////////////////////////////
  // CONSTRUCTOR            //
  ////////////////////////////
  /**
   *  Creates an {@code InterpolatedInteger} object with the specified integer value.
   *
   *  @param iValue The integer value that should be represented by this {@code InterpolatedInteger} object.
   **/
  public InterpolatedInteger(int iValue) {
    m_iValue = iValue;
  }
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the integer value that is represented by this object.
   *
   *  @return The integer value that is represented by this object.
  **/
  public int intValue() {
    return m_iValue;
  }
  
  
  /**
   *  Returns the integer value that is represented by this object as a double value.
   *
   *  @return The integer value that is represented by this object as a double value.
  **/
  public double doubleValue() {
    return (double)m_iValue;
  }
  
  /**
   *  Returns the integer value that is represented by this object as a string.
   *
   *  @return The integer value that is represented by this object as a string.
  **/
  public String toString() {
    return (m_iValue + "");
  }

  
  /////////////////////////////////
  // IINTERPOLATED INTERFACE     //
  /////////////////////////////////
  /**
   *  Returns an {@code InterpolatedInteger} object representing the integer value which is an interpolation between the integer value of this object, 
   *  and the integer value that has been passed on as parameter. 
   *
   *  @param pOtherObject The second integer value that is needed to calculate the interpolated value. 
   *  @param dLambda Steers the weights of the integer values. A value of 1 means that the value of this object gets fully considered. 
   *                 A value of 0 means that the parameter value gets fully considered. (TO DO: Should not it be the other way round?)
   *
   *  @return The resulting integer value of the interpolation.
   **/
  public InterpolatedInteger interpolateLinearly(InterpolatedInteger pOtherObject, double dLambda) {
    //1. make a simple check
    if (pOtherObject == null) return new InterpolatedInteger(m_iValue);
  
    //2. interpolate
    double dInterpolatedValue = (dLambda)*((double)m_iValue) + (1 -dLambda)*pOtherObject.doubleValue();
    return new InterpolatedInteger((int)Math.rint(dInterpolatedValue)); //round to the nearest integer
  }

}