package transparency;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.AlphaComposite;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.image.ExampleImageGenerator;

import static transparency.ImageTransitionParameters.*;

/**
 *  Implements a color transition effect.
 **/
public class ImageTransition extends ImageEffect {

  /*** private member variables ***/
  private ImageTransitionParameters m_pParameters;
  
  private BufferedImage m_pResultImage;
  private BufferedImage m_pImage1, m_pImage2;
  private float m_fStartX, m_fStartY, m_fEndX, m_fEndY, m_fFiftyPercentPos;
  
  /*** constants ****/
  private static final Color[] TRANSPARENCY_COLORS = new Color[] {
    new Color(0, 0, 0, 255), //opaque
    new Color(0, 0, 0, 128), //half transparent
    new Color(0, 0, 0, 0) //transparent
  };

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   * Creates a color transition effect object with the default effect settings.
   **/
  public ImageTransition() {
    this(new ImageTransitionParameters());
  }
  

  /**
   *   Creates a ColorTransition object with the specified parameters.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public ImageTransition(ImageTransitionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new ImageTransitionParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the color transition effect.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public void setImageTransitionParameters(ImageTransitionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
     
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the color transition effect. The parameters are only set, 
   *   if the object is of the type {@code ColorTransitionParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    //TODO solve this with reflection somehow?
    if (pEffectParameters instanceof ImageTransitionParameters) {
      setImageTransitionParameters((ImageTransitionParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *   Returns the parameters that are currently used for the grayscale reduction effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image processed with the current color transition.
   *
   *  @return An image processed with the current color transition.
   *
   */
  public BufferedImage process() throws Exception {
    //1. initial checks
    if ((m_pImage1 == null) && (m_pImage2 == null)) {
      return (new ExampleImageGenerator()).getImage();
    }
    
    if (m_pImage1 == null) return m_pImage2;
    if (m_pImage2 == null) return m_pImage1;
    
    
    
    
    
    //2. copy the background image into the result image
    int iWidth = m_pImage1.getWidth();
    int iHeight = m_pImage1.getHeight();
    m_pResultImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_ARGB);
    Graphics2D pGraphics = m_pResultImage.createGraphics();
    pGraphics.drawImage(m_pImage1, 0, 0, null);
    
    //3. create the gradient image with the foreground image and the opacity
    BufferedImage pGradientImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_ARGB);
    Graphics2D pGradientGraphics = pGradientImage.createGraphics();
    pGradientGraphics.drawImage(m_pImage2, 0, 0, null);
    
    float[] pFractions = new float[] {0.0f, m_fFiftyPercentPos, 1.0f};
    LinearGradientPaint pPaint = new LinearGradientPaint(m_fStartX, m_fStartY, m_fEndX, m_fEndY, pFractions, TRANSPARENCY_COLORS);
    pGradientGraphics.setComposite(AlphaComposite.DstIn);
    pGradientGraphics.setPaint(pPaint);
    pGradientGraphics.fillRect(0, 0, iWidth, iHeight);
   
    //4. add the gradient image to the result image
    pGraphics.drawImage(pGradientImage, 0, 0, null);
    

    return m_pResultImage;
    
    
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  
  
  
  private void fetchParameters() {
    
    m_pImage1 = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE1);
    m_pImage2 = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE2);
    
    m_fStartX = (float)m_pParameters.getIntegerParameterValue(PARAM_INT_TRANSITION_START_X);
    m_fStartY = (float)m_pParameters.getIntegerParameterValue(PARAM_INT_TRANSITION_START_Y);
    m_fEndX = (float)m_pParameters.getIntegerParameterValue(PARAM_INT_TRANSITION_END_X);
    m_fEndY = (float)m_pParameters.getIntegerParameterValue(PARAM_INT_TRANSITION_END_Y);
    m_fFiftyPercentPos = (float)m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS);
    
    
    
  }
  
  
  
  
  
  

}
