package transparency;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;
import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.IntegerParameter;
import standard.DoubleParameter;
import standard.ObjectParameter;
import standard.LinearTransitionProperties;
import util.image.ExampleImageGenerator;

/**
 *   Encapsulates the parameters that can be specified for the {@code ImageTransition} effect object. 
 **/
public class ImageTransitionParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/


  /*** KEY for the integer parameter: transition start x position in pixels  ***/
  public static final String PARAM_INT_TRANSITION_START_X = "TRANSITION_START_X";
  
  /*** KEY for the integer parameter: transition start y position in pixels  ***/
  public static final String PARAM_INT_TRANSITION_START_Y = "TRANSITION_START_Y";
  
  /*** KEY for the integer parameter: transition end x position in pixels  ***/
  public static final String PARAM_INT_TRANSITION_END_X = "TRANSITION_END_X";
  
  /*** KEY for the integer parameter: transition end y position in pixels  ***/
  public static final String PARAM_INT_TRANSITION_END_Y = "TRANSITION_END_Y";
  
  /*** KEY for the integer parameter: transition end y position in pixels  ***/
  public static final String PARAM_DOUBLE_TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS = "TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS";
  
  /*** KEY for the object parameter: the first image ***/
  public static final String PARAM_OBJECT_IMAGE1 = "IMAGE1";
  
  
  /*** KEY for the object parameter: the second image ***/
  public static final String PARAM_OBJECT_IMAGE2 = "IMAGE2";

    
   /***
   *   Creates a default parameter set for the {@code ColorTransition} effect. The following settings are used:
   *
   *   <ul>
   *     <li>A transition length of 80 pixels.</li>
   *     <li>A transition end color of rgb(50, 50, 255)</li>
   *     <li>An empty image with the dimension 400 x 400 pixels</li>
   *     <li>A transition to the bottom is activated.</li>
   *   </ul>
   *
   **/
  public ImageTransitionParameters() {

    //1. create the parameters and set them
    setIntegerParameters(createIntegerParameters());
    setDoubleParameters(createDoubleParameters());
    setObjectParameters(createObjectParameters());

  }
  
  
  ////////////////////////////////////////////
  // GET METHODS                            //
  ////////////////////////////////////////////
  public LinearTransitionProperties getLinearTransitionProperties() {
    return new LinearTransitionProperties(
      getIntegerParameterValue(PARAM_INT_TRANSITION_START_X), 
      getIntegerParameterValue(PARAM_INT_TRANSITION_START_Y), 
      getIntegerParameterValue(PARAM_INT_TRANSITION_END_X), 
      getIntegerParameterValue(PARAM_INT_TRANSITION_END_Y), 
      getDoubleParameterValue(PARAM_DOUBLE_TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS)
    );
  }
  
  
  ////////////////////////////////////////////
  // SET METHODS                            //
  ////////////////////////////////////////////
  public  void setLinearTransitionProperties(LinearTransitionProperties pProperties) {
    if (pProperties == null) return;
    setIntegerParameterValue(PARAM_INT_TRANSITION_START_X, pProperties.getStartX());
    setIntegerParameterValue(PARAM_INT_TRANSITION_START_Y, pProperties.getStartY());
    setIntegerParameterValue(PARAM_INT_TRANSITION_END_X, pProperties.getEndX());
    setIntegerParameterValue(PARAM_INT_TRANSITION_END_Y, pProperties.getEndY());
    setDoubleParameterValue(PARAM_DOUBLE_TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS, pProperties.getFiftyPercentTranparencyPos());
  }
  
  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

   
    //2. fill the hashtable
    pParameters.put(PARAM_INT_TRANSITION_START_X, new IntegerParameter(0, -50000, 50000));
    pParameters.put(PARAM_INT_TRANSITION_START_Y, new IntegerParameter(200, -50000, 50000));
    pParameters.put(PARAM_INT_TRANSITION_END_X, new IntegerParameter(400, -50000, 50000));
    pParameters.put(PARAM_INT_TRANSITION_END_Y, new IntegerParameter(200, -50000, 50000));
    
    
    //3. return the hashtable
    return pParameters;
  }
  
  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

   
    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_TRANSITION_FIFTY_PERCENT_TRANSPARENCY_POS, new DoubleParameter(0.5, 0, 1));
    
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  
  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    
    //2. fill the hashtable
    ExampleImageGenerator pGenerator = new ExampleImageGenerator();
    pGenerator.setLeftUpperColor(Color.RED);
    pGenerator.setRightLowerColor(Color.RED);
    pParameters.put(PARAM_OBJECT_IMAGE1, new ObjectParameter(pGenerator.getImage()));
    
    pGenerator.setLeftUpperColor(Color.BLUE);
    pGenerator.setRightLowerColor(Color.BLUE);
    pParameters.put(PARAM_OBJECT_IMAGE2, new ObjectParameter(pGenerator.getImage()));
    
    
  
    //3. return the hashtable
    return pParameters;
  }
  
  
  
  
}
  
  
  
