package util.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;
import java.util.Collections;
import java.awt.Color;


/**
 *  Represents a transition between n colored positions on the [0,1] interval.
 **/  
public class ColorGradient {

  /*** member variables ***/
  private Vector<ColorPoint1D> m_pColorPoints;
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates a default color gradient object having the color black at position 0 and white at position 1. 
   *
   **/
  public ColorGradient() {
    m_pColorPoints = new Vector<ColorPoint1D>();
    m_pColorPoints.add(new ColorPoint1D(Color.BLACK, 0));
    m_pColorPoints.add(new ColorPoint1D(Color.WHITE, 1));
    
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the position of the specified color point. 
   *
   *  @param iColorPointId The id of the color point as integer. 
   *  @param dPosition The new position of the color point as double between 0 and 1. 
   *                   If the position is outside this interval it is mapped to the nearest interval limit.
   **/
  public void setPositionOfColorPoint(int iColorPointId, double dPosition) {
    //1. make some checks
    if (iColorPointId < 0) return;
    if (iColorPointId >= m_pColorPoints.size()) return;
    
    if (dPosition < 0) dPosition = 0;
    if (dPosition > 1) dPosition = 1;
    
    //2. modify the specified color point
    m_pColorPoints.get(iColorPointId).setPosition(dPosition);
    Collections.sort(m_pColorPoints);
  }
  
  /**
   *  Sets the color of the specified color point. 
   *
   *  @param iColorPointId The id of the color point as integer. 
   *  @param pColor The new color of the color point.
   **/
  public void setColorOfColorPoint(int iColorPointId, Color pColor) {
    //1. make some checks
    if (iColorPointId < 0) return;
    if (iColorPointId >= m_pColorPoints.size()) return;
    if (pColor == null) return;
    
    //2. modify the specified color point
    m_pColorPoints.get(iColorPointId).setColor(pColor);
  }
  
  /**
   *  Removes the specified color point.
   *
   *  @param iColorPointId The id of the color point as integer. 
   **/
  public void removeColorPoint(int iColorPointId) {
    //1. make some checks
    if (iColorPointId < 0) return;
    if (iColorPointId >= m_pColorPoints.size()) return;
    if (m_pColorPoints.size() == 2) return;
    
    //2. remove the color point
    m_pColorPoints.remove(iColorPointId);
     
  }
  
  /**
   *  Adds a color point with a specified color and position.
   *
   *  @param pColor The new color of the color point.
   *  @param dPosition The new position of the color point as double between 0 and 1. 
   *                   If the position is outside this interval it is mapped to the nearest interval limit.
   **/
  public void addColorPoint(Color pColor, double dPosition) {
    //1. make some checks
    if (dPosition < 0) dPosition = 0;
    if (dPosition > 1) dPosition = 1;
    if (pColor == null) return;
    
    //2. add the point
    ColorPoint1D pColorPoint = new ColorPoint1D(pColor, dPosition);
    m_pColorPoints.add(pColorPoint);
    Collections.sort(m_pColorPoints);
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////

  
  /**
   *  Returns the id of the first color point having the specified color. If no color point could be found with that color, -1 is returned.
   *
   *  @param pColor The color of the color point that should be found.
   *
   *  @return The id of the first color point having the specified color. If no color point could be found with that color, -1 is returned.
   **/
  public int getIdOfColorPointWithColor(Color pColor) {
    if (pColor == null) return -1;
    
    int iColorPointNum = m_pColorPoints.size();
    for (int i = 0; i < iColorPointNum; i++) {
      if (m_pColorPoints.get(i).getColor().equals(pColor)) {
        return i;
      }
    }
    
    return -1;
  }
  
  /**
   *  Returns the current number of color points.
   *
   *
   *  @return The current number of color points.
   **/
  public int getColorPointNum() {
    return m_pColorPoints.size();
  }
  
  
  /**
   *  Returns the position of the specified color point.
   *
   *  @param iColorPointId The id of the color point as integer. 
   *
   *  @return The position of the specified color point.
   **/
  public double getPositionOfColorPoint(int iColorPointId) {
    //1. make some checks
    if (iColorPointId < 0) return 0;
    if (iColorPointId >= m_pColorPoints.size()) return 1;

    
    //2. return the color of the specified color point
    return m_pColorPoints.get(iColorPointId).getPosition();
  
  }
  
  /**
   *  Returns the color of the specified color point.
   *
   *  @param iColorPointId The id of the color point as integer. 
   *
   *  @return The color of the specified color point.
   **/  
  public Color getColorOfColorPoint(int iColorPointId) {
    //1. make some checks
    if (iColorPointId < 0) return null;
    if (iColorPointId >= m_pColorPoints.size()) return null;

    
    //2. return the color of the specified color point
    return m_pColorPoints.get(iColorPointId).getColor();
  }
  
  /**
   *  Returns the color at the specified position.
   *
   *  @param dPosition The position as double betweent 0 and 1. 
   *
   *  @return The color at the specified position.
   **/  
  public Color getColorAtPosition(double dPosition) {
    return new Color(getRGBAtPosition(dPosition));
  }
  
  /**
   *  Returns the color at the specified position.
   *
   *  @param dPosition The position as double betweent 0 and 1. 
   *
   *  @return The color at the specified position as combined RGB value consisting of the red component 
   *          in bits 16-23, the green component in bits 8-15, and the blue component in bits 0-7.
   **/  
  public int getRGBAtPosition(double dPosition) {
    
    //2. return the color
    //check the borders
    if (dPosition <= m_pColorPoints.get(0).getPosition()) {
      return m_pColorPoints.get(0).getColor().getRGB();
    }
    
    int iColorPointNum = m_pColorPoints.size();
    if (dPosition >= m_pColorPoints.get(iColorPointNum-1).getPosition()) {
      return m_pColorPoints.get(iColorPointNum-1).getColor().getRGB(); 
    }
    
    //find the neighbouring colors
    ColorPoint1D pLeftNeighbour = null;
    ColorPoint1D pRightNeighbour = null;
    
    for (int i = 0; i < iColorPointNum-1; i++) {
      if ((m_pColorPoints.get(i).getPosition() <= dPosition) &&
          (m_pColorPoints.get(i+1).getPosition() > dPosition)) {
        pLeftNeighbour = m_pColorPoints.get(i);
        pRightNeighbour = m_pColorPoints.get(i+1);
        break;
      }
    }    
  
    //linear interpolation
    if (pLeftNeighbour == null) return 0;
    if (pRightNeighbour == null) return 0;
    double dLeftPos = pLeftNeighbour.getPosition();
    double dRightPos = pRightNeighbour.getPosition();
    double dLambda = (dPosition-dLeftPos)/(dRightPos-dLeftPos);
    
    Color pLeftCol = pLeftNeighbour.getColor();
    Color pRightCol = pRightNeighbour.getColor();
    
    double dRed = pLeftCol.getRed()*(1-dLambda) + dLambda*pRightCol.getRed();
    double dGreen = pLeftCol.getGreen()*(1-dLambda) + dLambda*pRightCol.getGreen();
    double dBlue = pLeftCol.getBlue()*(1-dLambda) + dLambda*pRightCol.getBlue();
  
    if (dRed > 255) dRed = 255;
    if (dRed < 0) dRed = 0;    
    if (dGreen > 255) dGreen = 255;
    if (dGreen < 0) dGreen = 0;
    if (dBlue > 255) dBlue = 255;
    if (dBlue < 0) dBlue = 0;
  
    return ((255 << 24) | ((int)dRed << 16) | ((int)dGreen << 8) | (int)dBlue);
  }
  
  
  
  
  /**
   *    Returns a color lookup table with the specified number of entries, 
   *    where the entry 0 represents the color at position 0 and the n-th entry 
   *    represents the color at position 1. The colors are represented by their 
   *    RGB int values. 
   *
   *    @param iEntryNum The number of entries that the color lookup table should have.
   *
   *    @return The color lookup table as integer array holding the colors as combined RGB value consisting of the red component 
   *            in bits 16-23, the green component in bits 8-15, and the blue component in bits 0-7.
   **/
  public int[] getRGBLookupTable(int iEntryNum) {
    //1. make a simple check
    if (iEntryNum < 2) iEntryNum = 2;
    
    //2. create the color array and fill it
    int[] pLookupTable = new int[iEntryNum];
    double dIncrease = 1.0/(double)(iEntryNum-1);
    double dCurrentPos = 0;
    
    for (int i = 0; i < iEntryNum; i++) {
      
      pLookupTable[i] = getRGBAtPosition(dCurrentPos);
      dCurrentPos += dIncrease;
    }
    
    //3. return the lookup table
    return pLookupTable;
  }
  
}
