package util.math;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

 /**
  *  Encapsulates a complex number and provides also methods to calculate with them. 
  **/
public class ComplexNumber {

  /*** public member variables ***/
  /*** The real part of the complex number as double. ***/
  public double m_dReal;
  
  /*** The imaginary part of the complex number as double. ***/
  public double m_dImaginary;
  
  ///////////////////////
  // CONSTRUCTOR       //
  ///////////////////////
  /**
   *  Creates a complex number with the specified real and imaginary part. 
   *
   *  @param dReal The real part of the complex number as double.
   *  @param dImaginary The imaginary part of the complex number as double.
   **/
  public ComplexNumber(double dReal, double dImaginary) {
    m_dReal = dReal;
    m_dImaginary = dImaginary;
  }
  
  public ComplexNumber(ComplexNumber pComplexNumber) {
    if (pComplexNumber == null) {
      m_dReal = 0;
      m_dImaginary = 0;
    }
    else {
      m_dReal = pComplexNumber.m_dReal;
      m_dImaginary = pComplexNumber.m_dImaginary;
    }
  }
  
  
  ///////////////////////
  // GET METHODS       //
  ///////////////////////
  /**
   *  Returns the length of the complex number (i.e. the length of the vector in the complex plane). 
   *
   *  @return The length of the complex number. 
   **/
  public double getLength() {
    return Math.sqrt(m_dReal*m_dReal + m_dImaginary*m_dImaginary);
  }
  
  public String toString() {
    return m_dReal + " + " + m_dImaginary + "i";
  }
  
  //////////////////////////
  // COMPARISON           //
  //////////////////////////
  /**
   *  Returns true, if the specified complex number is equal to this object.
   *
   *  @param pOtherNumber The other complex number that should be compared.
   *
   *  @return true, if the specified complex number is equal to this object.
   **/
  public boolean equals(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return false;
    if ((pOtherNumber.m_dReal == m_dReal) && (pOtherNumber.m_dImaginary == m_dImaginary)) {
      return true;
    }
    return false;
  }
  
  ///////////////////////
  // OPERATIONS        //
  ///////////////////////
  /**
   *  Returns A complex number that is the sum between this complex number and the specified parameter. 
   *  The values of this object are not modified!
   *
   *  @param pOtherNumber The other complex number that should be added to the values of this object.
   *
   *  @return A complex number that is the sum between this complex number and the specified parameter.
   **/
  public ComplexNumber add(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return new ComplexNumber(m_dReal, m_dImaginary);
    double dReal = pOtherNumber.m_dReal + m_dReal;
    double dImaginary = pOtherNumber.m_dImaginary + m_dImaginary;
    return new ComplexNumber(dReal, dImaginary);
  }
  
  /**
   *  Adds the values of the specified complex number to this complex number.
   *
   *  @param pOtherNumber The other complex number that should be added to the values of this object.
   *
   **/
  public void addToThis(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return;
    m_dReal += pOtherNumber.m_dReal;
    m_dImaginary += pOtherNumber.m_dImaginary;
  }
  
  /**
   *  Returns A complex number that is the result of this complex number subtracted by the specified complex number.
   *  The values of this object are not modified!
   *
   *  @param pOtherNumber The other complex number that should be subtracted from the values of this object.
   *
   *  @return A complex number that is the result of this complex number subtracted by the specified complex number.
   **/
  public ComplexNumber subtract(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return new ComplexNumber(m_dReal, m_dImaginary);
    double dReal = m_dReal - pOtherNumber.m_dReal;
    double dImaginary = m_dImaginary - pOtherNumber.m_dImaginary;
    return new ComplexNumber(dReal, dImaginary);
  }
  
  /**
   *  Subtracts the values of the specified complex number from this complex number.
   *
   *  @param pOtherNumber The other complex number that should be subtracted from the values of this object.
   *
   **/
  public void subtractFromThis(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return;
    m_dReal -= pOtherNumber.m_dReal;
    m_dImaginary -= pOtherNumber.m_dImaginary;
  }
  
  
  /**
   *  Returns A complex number that is the result of this complex number multiplied by the specified complex number.
   *  The values of this object are not modified!
   *
   *  @param pOtherNumber The other complex number that should be multiplied with the values of this object.
   *
   *  @return A complex number that is the result of this complex number multiplied by the specified complex number.
   **/
  public ComplexNumber multiply(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return new ComplexNumber(m_dReal, m_dImaginary);
    double dReal = m_dReal*pOtherNumber.m_dReal - m_dImaginary*pOtherNumber.m_dImaginary;
    double dImaginary = m_dReal*pOtherNumber.m_dImaginary + m_dImaginary*pOtherNumber.m_dReal;
    return new ComplexNumber(dReal, dImaginary);
  }
  
  /**
   *  Multiplies the values of the specified complex number with this complex number.
   *
   *  @param pOtherNumber The other complex number that should be multiplied with the values of this object.
   *
   **/
  public void multiplyWithThis(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return;
    
    double dReal = m_dReal*pOtherNumber.m_dReal - m_dImaginary*pOtherNumber.m_dImaginary;
    double dImaginary = m_dReal*pOtherNumber.m_dImaginary + m_dImaginary*pOtherNumber.m_dReal;
    
    m_dReal = dReal;
    m_dImaginary = dImaginary;
  }
  
  /**
   *  Returns A complex number that is the result of this complex number divided by the specified complex number.
   *  The values of this object are not modified!
   *
   *  @param pOtherNumber The other complex number that should be used to divide this object.
   *
   *  @return A complex number that is the result of this complex number divided by the specified complex number.
   **/
  public ComplexNumber divideBy(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return new ComplexNumber(m_dReal, m_dImaginary);
    
    double dDivisor = pOtherNumber.m_dReal*pOtherNumber.m_dReal;
    dDivisor += pOtherNumber.m_dImaginary*pOtherNumber.m_dImaginary;
    if (dDivisor == 0) return new ComplexNumber(m_dReal, m_dImaginary);
    
    double dReal = m_dReal*pOtherNumber.m_dReal + m_dImaginary*pOtherNumber.m_dImaginary;
    double dImaginary = m_dReal*pOtherNumber.m_dImaginary + m_dImaginary*pOtherNumber.m_dReal;
    
    dReal /= dDivisor;
    dImaginary /= dDivisor;
    
    return new ComplexNumber(dReal, dImaginary);
  }
  
  /**
   *  Divides this complex number by the specified complex number.
   *
   *  @param pOtherNumber The other complex number that should be used to divide this object.
   *
   **/
  public void divideThisBy(ComplexNumber pOtherNumber) {
    if (pOtherNumber == null) return;
    
    double dDivisor = pOtherNumber.m_dReal*pOtherNumber.m_dReal;
    dDivisor += pOtherNumber.m_dImaginary*pOtherNumber.m_dImaginary;
    if (dDivisor == 0) return;
    
    double dReal = m_dReal*pOtherNumber.m_dReal + m_dImaginary*pOtherNumber.m_dImaginary;
    double dImaginary = m_dReal*pOtherNumber.m_dImaginary + m_dImaginary*pOtherNumber.m_dReal;
    
    dReal /= dDivisor;
    dImaginary /= dDivisor;
    
    m_dReal = dReal;
    m_dImaginary = dImaginary;
    
  }
  
  

}
